/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AVEncoder.h,v 1.1.1.1 2003/12/24 19:41:39 gadibergman Exp $
 */

/************************************************************
				AVEncoder.h
				===========

  An object responsible of integrating:
  - audio input
  - video input
  - an audio encoder
  - a video encoder
  - an AV-Format writer

  This object uses
  - AV Format writer -- FFMPEG implementation of AV Format writer
  - audio encoder    -- whatever implements CAudioEncoder
  - video encoder    -- FFMPEG implementation of MPEG-4 Video
                        Used internally by AV Format writer

************************************************************/

#if !defined(AFX_AVENCODER_H__39A1711C_04D5_4479_8D24_2C99E8F63816__INCLUDED_)
#define AFX_AVENCODER_H__39A1711C_04D5_4479_8D24_2C99E8F63816__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MPEGEncoder.h"

class TimeKeepingWriter;
class CDXVideoReceiver;
class CDXSoundReceiver;
class CTimeBasedPipe;
class CAudioEncoder;
class CAudioEncoderGenerator;
class RGB2YUVConvertor;
class CXVidEncoder;
class CAVFormatWriter;
class CFileNameGenerator;
class CRGBBlockCopier;
class CVideoCodecTrackWriter;
class CMediaProvider;

class CAVEncoder : public CMPEGEncoder
{
private:
	TimeKeepingWriter *m_pAudioTrackWriter;
	CVideoCodecTrackWriter *m_pVideoTrackWriter;

	CDXSoundReceiver * m_pSoundReceiver;
	CDXVideoReceiver * m_pVideoReceiver;

	CAudioEncoder *m_pAudioEncoder;
	CRGBBlockCopier *m_pRGBBlockCopier;
	CXVidEncoder *m_pXVidEncoder;
	CAVFormatWriter *m_pAVFormatWriter;

	CMediaProvider * m_pAudioProvider;
	CMediaProvider * m_pVideoProvider;

	CAudioEncoderGenerator & m_AudioEncoderGenerator;
	CFileNameGenerator & m_FileNameGenerator;

	unsigned int m_uDesiredFrameRate;

	CTimeBasedPipe
		*m_pPCMAudioPipe,
		*m_pRGBVideoPipe,
		*m_pYUVVideoPipe;

	void Init();


public:
	void GetProgress(
		double&	time,
		LONGLONG& pcmBytes, 
		LONGLONG& aacBytes, 
		LONGLONG& rgbBytes, 
		LONGLONG& yuvBytes, 
		LONGLONG& xvidBytes
		) const;

	CAVEncoder(
		CDXSoundReceiver * pSoundReceiver,
		CDXVideoReceiver * pVideoReceiver,
		CAudioEncoderGenerator & audioEncoderGenerator,
		CFileNameGenerator & fileNameGenerator,
		unsigned int uDesiredFrameRate
		):
	m_pSoundReceiver(pSoundReceiver),
	m_pVideoReceiver(pVideoReceiver),
	m_AudioEncoderGenerator(audioEncoderGenerator),
	m_FileNameGenerator(fileNameGenerator),
	m_uDesiredFrameRate(uDesiredFrameRate),
	m_pPCMAudioPipe(NULL),
	m_pRGBVideoPipe(NULL),
	m_pYUVVideoPipe(NULL),
	m_pAudioTrackWriter(NULL),
	m_pVideoTrackWriter(NULL),
	m_pAudioEncoder(NULL),
	m_pRGBBlockCopier(NULL),
	m_pXVidEncoder(NULL),
	m_pAVFormatWriter(NULL),
	m_pAudioProvider(NULL),
	m_pVideoProvider(NULL)
	{
		Init();
	}

	void Stop();
	void PlayOrPause();

	virtual ~CAVEncoder();

};

#endif // !defined(AFX_AVENCODER_H__39A1711C_04D5_4479_8D24_2C99E8F63816__INCLUDED_)
