/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AVFormatWriter.h,v 1.1.1.1 2003/12/24 19:41:42 gadibergman Exp $
 */

/************************************************************
				AVFormatWriter.h
				================

  The combination of two channels - one video and one audio
  into one file, such as Microsoft's AVI.

  - Uses FFMPEG library project (libmedia.dll).
  - Uses Timestamp and Duration that are imported 
	from MPEG4IP project.
************************************************************/

#if !defined(AFX_AVFORMATWRITER_H__A4AB0A73_706F_47C8_99D3_540EE5F0EBA7__INCLUDED_)
#define AFX_AVFORMATWRITER_H__A4AB0A73_706F_47C8_99D3_540EE5F0EBA7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "mpeg4/combined/media_frame.h"

// Project classes
class FFMPEGAccess;
class TimeKeepingWriter;
class CTimeBasedProgressSelection;
class CVideoCodecTrackWriter;

// FFMPEG structures
struct AVFormatContext;
struct AVOutputFormat;
struct AVStream;

class CAVFormatWriter {
private:
	CTimeBasedProgressSelection* m_pProgressSelection;

	bool m_bAudioDone, m_bVideoDone;

	unsigned int
		m_uAudioSampleRate,
		m_uAudioChannels,
		m_uDesiredFrameRate;

	unsigned short
		m_uFrameWidth,
		m_uFrameHeight;

	double
		m_dFrameRate;

	unsigned long
		m_uAudioBitRate,
		m_uVideoFramesNumber;

	Timestamp
		m_tsRecent;

	
public:
	void DeclareWritingEnded(CodecType codecType);
	void WriteFrame(CodecType codecType, const void* buf, unsigned int numBytes,
		Timestamp timestamp, Duration duration);

	TimeKeepingWriter* CreateTrackWriter(CodecType codecType, CodecID codecID);
	CVideoCodecTrackWriter* CreateVideoCodecTrackWriter(CodecID codecID);

	void Start();

	CAVFormatWriter(
		unsigned int uAudioSampleRate,
		unsigned int uAudioChannels,
		unsigned long uAudioBitRate,
		unsigned short uFrameWidth,
		unsigned short uFrameHeight,
		double dFrameRate,
		unsigned int uDesiredFrameRate,
		LPCTSTR sFileName);

	virtual ~CAVFormatWriter();

	double GetTime() const
	{
		return ((double) (__int64) m_tsRecent) / TimestampTicks;
	}

private:
	void Stop();
	CString m_sFileName;
	FFMPEGAccess& m_ffmpeg;
    AVOutputFormat *m_pFormat;
    AVFormatContext *m_pContext;
	AVStream *m_pAudioStream, *m_pVideoStream;
};

#endif // !defined(AFX_AVFORMATWRITER_H__A4AB0A73_706F_47C8_99D3_540EE5F0EBA7__INCLUDED_)
