/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ActiveApplicationObjects.h,v 1.2 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
				ActiveApplicationObjects.h
				==========================

  A functional layer that stores pointers to active UI 
  objects - the video viewer and the audio recorder dialog.

************************************************************/

#if !defined(AFX_ACTIVEAPPLICATIONOBJECTS_H__D0612CDC_E7CB_4E4E_A214_FA211C0B1CC9__INCLUDED_)
#define AFX_ACTIVEAPPLICATIONOBJECTS_H__D0612CDC_E7CB_4E4E_A214_FA211C0B1CC9__INCLUDED_

// Classes declarations
class CMPEG4AVRecorder;
class CRecorderDlg;
class CVideoViewer;
class CVolumeMeterDialog;

// Includes
#include "TimeDependentFileNameGenerator.h"
#include "SpecificApplicationData.h"
#include "AudioEncoderGenerator.h"
#include "Common/MutexProtection.h"
#include "RepeatingTimeEvent.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class CActiveApplicationObjects  
{
public:
	void DisplayVolumeUnitMeter();
	void OnTimeChanged();
	void OnMinutePassed();
	void ManageRecordingEvents();
	void RecordVideo(bool bIsAutomatic);
	void ShowMyFiles();
	void WatchVideo(bool bIsManual);
	void RecordAudio();
	bool LoadApplicationData();
	CActiveApplicationObjects();
	virtual ~CActiveApplicationObjects();

	bool UseProperties();
	bool EditProperties();

private:
	void UpdateNextRecordingEvent();

	IBaseFilter* m_pVideoFilter;
	GUID m_gRecordingDeviceID;
	CTimeDependentFileNameGenerator 
		m_AudioFileNameGenerator, m_MPEGFileNameGenerator;
	CAudioEncoderGenerator m_AudioEncoderGenerator;
	CSpecificApplicationData m_SpecificApplicationData;
	CMPEG4AVRecorder * m_pCOMAVRecorder;

	CTimeEventsVector m_vTimeEvents;
	bool m_bHasNextEvent, m_bNextEventIsEnding;
	CSimpleTimePoint m_tpNextEvent;

	bool m_bInTransition;
	HANDLE m_hDevicesUsageMutex;
	CVideoViewer * m_pVideoViewer;
	CRecorderDlg * m_pAudioRecorderDlg;
	CVolumeMeterDialog* m_pVolumeMeterDlg;
};

#endif // !defined(AFX_ACTIVEAPPLICATIONOBJECTS_H__D0612CDC_E7CB_4E4E_A214_FA211C0B1CC9__INCLUDED_)
