/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AudioEncoderGenerator.cpp,v 1.2 2004/03/27 13:47:16 gadibergman Exp $
 */

/************************************************************
				AudioEncoderGenerator.cpp
				=========================

  A way of generating an instance of an audio encoder
  according to the encoder type.

************************************************************/

// PCH includes
#include "stdafx.h"

// Project includes
#include "MPEGRecorder.h"
#include "AudioEncoderGenerator.h"
#include "FAACEncoder.h"
#include "LameEncoder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAudioEncoderGenerator::~CAudioEncoderGenerator()
{

}

CAudioEncoder* CAudioEncoderGenerator::
CreateAudioEncoder(
					TimeKeepingReader *pReader, 
					TimeKeepingWriter *pWriter, 
					unsigned short usChannels, 
					unsigned short usBytesPerSample, 
					unsigned short usSampleRate,
					EncoderContainerType ectContainer
				  ) const
{
	switch(m_CurrentType) {
	case LAME_MP3_ENCODER:
		return new CLameEncoder	(
									pReader, 
									pWriter,
									usChannels, 
									usBytesPerSample, 
									usSampleRate,
									m_ulKBitRate
								);

	case FAAC_AAC_ENCODER:
		{
			bool bIsMP4 = ( ectContainer == MP4_FILE_CONTAINER );
		return new CFAACEncoder	(	pReader, 
									pWriter,
									usSampleRate, 
									usChannels,
									m_ulKBitRate,
									bIsMP4
								);
		}

	}

	return NULL;
}

LPCTSTR CAudioEncoderGenerator::GetDefaultEncodedFileExtension() const
{
	switch(m_CurrentType) {
	case LAME_MP3_ENCODER:
		return _T(".mp3");

	case FAAC_AAC_ENCODER:
		return _T(".aac");

	}

	return NULL;
}

MediaType CAudioEncoderGenerator::GetMp4MediaFrameType() const
{
	switch (m_CurrentType) {
	case LAME_MP3_ENCODER:
		return MP3AUDIOFRAME;

	case FAAC_AAC_ENCODER:
		return AACAUDIOFRAME;

	}

	return 0;
}

CodecID CAudioEncoderGenerator::GetCodecID() const
{
	switch (m_CurrentType) {
	case LAME_MP3_ENCODER:
		return CODEC_ID_MP3LAME;

	case FAAC_AAC_ENCODER:
		return CODEC_ID_AAC;	

	}

	return CODEC_ID_NONE;
}
