/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: AudioEncoderGenerator.h,v 1.2 2004/03/27 13:47:16 gadibergman Exp $
 */

/************************************************************
				AudioEncoderGenerator.h
				=======================

  A way of generating an instance of an audio encoder
  according to the encoder type.

************************************************************/

#if !defined(AFX_AUDIOENCODERGENERATOR_H__8FC111ED_BC91_4ACA_BDD9_3B84AD4B12F5__INCLUDED_)
#define AFX_AUDIOENCODERGENERATOR_H__8FC111ED_BC91_4ACA_BDD9_3B84AD4B12F5__INCLUDED_

#include "MPEG4\COMBINED\media_frame.h"	// Added by ClassView
#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

class TimeKeepingReader;
class TimeKeepingWriter;
class CAudioEncoder;

class CAudioEncoderGenerator  
{
public:
	virtual ~CAudioEncoderGenerator();

	enum AudioEncoderType {
		UNDEFINED_ENCODER = -1,
		LAME_MP3_ENCODER = 0,
		FAAC_AAC_ENCODER = 1
	};

	enum EncoderContainerType {
		SIMPLE_FILE_CONTAINER,
		MP4_FILE_CONTAINER,
		AVI_FILE_CONTAINER
	};

	CAudioEncoder* CreateAudioEncoder(
		TimeKeepingReader * pReader, 
		TimeKeepingWriter * pWriter, 
		unsigned short usChannels, 
		unsigned short usBytesPerSample, 
		unsigned short usSampleRate,
		EncoderContainerType ectContainer 
		) const;

private:
	AudioEncoderType m_CurrentType;
	unsigned long m_ulKBitRate;

public:
	CodecID GetCodecID() const;
	MediaType GetMp4MediaFrameType() const;

	LPCTSTR GetDefaultEncodedFileExtension() const;

	CAudioEncoderGenerator(unsigned long ulKBitRate):
	m_CurrentType(UNDEFINED_ENCODER),
	m_ulKBitRate(ulKBitRate)
	{}

	unsigned long GetBitRate() const {
		return 1000 * m_ulKBitRate;
	}

	void SetKBitRate(unsigned long ulKBitRate) {
		m_ulKBitRate = ulKBitRate;
	}

	void SetEncoderType(AudioEncoderType type) {
		m_CurrentType = type;
	}
};

#endif // !defined(AFX_AUDIOENCODERGENERATOR_H__8FC111ED_BC91_4ACA_BDD9_3B84AD4B12F5__INCLUDED_)
