/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: CFileWriter.cpp,v 1.1.1.1 2003/12/24 19:41:37 gadibergman Exp $
 */

/************************************************************
				CFileWriter.cpp
				===============

  An abstraction of writing blocks to a file using MFC's 
  CFile object.

************************************************************/


// PCH include
#include "StdAfx.h"

// STL includes
#include <string>

// Project includes
#include "CFileWriter.h"

using namespace std;

unsigned int CFileWriter::Write(
	const void * buf, 
	unsigned int numBytes,
	Timestamp timestamp, 
	Duration duration
	) 
{

	try {
		m_pFile->Write(buf, numBytes);
	}
	catch (CFileException ex) {
      TCHAR   szCause[255];
      CString strFormatted;

      ex.GetErrorMessage(szCause, 255);

      // (in real life, it's probably more
      // appropriate to read this from
      //  a string resource so it would be easy to
      // localize)

      strFormatted = _T("The data file could not be opened because of this error: ");
      strFormatted += szCause;

	  throw new string((LPCTSTR) strFormatted);
	}

	return numBytes;
}

void CFileWriter::Close() {
	m_pFile->Close();
	m_bCloseOnDestruction = false;
}

CFileWriter::CFileWriter(const char * fileName):
m_pFile(new CFile(fileName, CFile::modeWrite | CFile::modeCreate)),
m_bCloseOnDestruction(true),
m_bDeleteOnDestruction(true)
{
}

CFileWriter::~CFileWriter() {
	if (m_bCloseOnDestruction)
		m_pFile->Close();
	if (m_bDeleteOnDestruction)
		delete m_pFile;
}

