/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: CFileWriter.h,v 1.1.1.1 2003/12/24 19:41:39 gadibergman Exp $
 */

/************************************************************
				CFileWriter.h
				=============

  An abstraction of writing blocks to a file using MFC's 
  CFile object.

************************************************************/

#ifndef __CFILE_WRITER
#define __CFILE_WRITER

// Includes
#include "TimeKeepingReaderWriter.h"

// Classes declaration
class CFile;

class CFileWriter : public TimeKeepingWriter {

private:
	CFile * m_pFile;
	bool m_bCloseOnDestruction;
	bool m_bDeleteOnDestruction;

protected:
	unsigned int Write(const void * buf, unsigned int numBytes,
		Timestamp timestamp, Duration duration);

public:

	CFileWriter(CFile * pFile):
	  m_pFile(pFile),
	  m_bCloseOnDestruction(false),
	  m_bDeleteOnDestruction(false)
	  {}

	CFileWriter(const char * fileName);

	virtual void DeclareWritingEnded() {
		TimeKeepingWriter::DeclareWritingEnded();
		if (m_bCloseOnDestruction)
			Close();
	}

	virtual ~CFileWriter();

	void Close();
};


#endif