/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MutexProtection.cpp,v 1.1.1.1 2003/12/24 19:45:11 gadibergman Exp $
 */

/************************************************************
			MutexProtection.cpp
			===================

  Synchronization objects, based on Windows synchronization
  functions.
  
************************************************************/

// VC++ shit
#include "StdAfx.h"

// Own header file include
#include "MutexProtection.h"


// --------------------------------------------------------
// CreateNewMutex
// --------------------------------------------------------
HANDLE CreateNewMutex() {
	return CreateMutex(NULL, FALSE, NULL);
}

// --------------------------------------------------------
// Mutex - constructor
// --------------------------------------------------------
Mutex::Mutex():
	  _mutexHandle(CreateNewMutex())
{}

// --------------------------------------------------------
// Mutex - destructor
// --------------------------------------------------------
Mutex::~Mutex() {
	CloseHandle(_mutexHandle);
}

// --------------------------------------------------------
// MutexProtection - constructor
// --------------------------------------------------------
MutexProtection::MutexProtection(const Mutex & mutex):
	_mutexHandle(mutex._mutexHandle) {
	DWORD dwResult = WaitForSingleObject(_mutexHandle, INFINITE);
	ASSERT(dwResult == WAIT_OBJECT_0);
}

MutexProtection::MutexProtection(HANDLE hMutex):
_mutexHandle(hMutex)
{
	DWORD dwResult = WaitForSingleObject(_mutexHandle, INFINITE);
	ASSERT(dwResult == WAIT_OBJECT_0);
}

// --------------------------------------------------------
// MutexProtection - destructor
// --------------------------------------------------------
MutexProtection::~MutexProtection() {
	BOOL bResult = ReleaseMutex(_mutexHandle);
	ASSERT(bResult);
}

// --------------------------------------------------------
// CreateNewSemaphore
// --------------------------------------------------------
HANDLE CreateNewSemaphore(uint initialCount, uint maxCount) {
	return CreateSemaphore(NULL, initialCount, maxCount, NULL);
}

// --------------------------------------------------------
// Semaphore - c'tor
// --------------------------------------------------------
Semaphore::Semaphore(uint initialCount, uint maxCount):
	_semHandle(CreateNewSemaphore(initialCount, maxCount))
	{}

// --------------------------------------------------------
// Semaphore - d'tor
// --------------------------------------------------------
Semaphore::~Semaphore() {
	CloseHandle(_semHandle);
}

// --------------------------------------------------------
// Semaphore::Wait
// --------------------------------------------------------
void Semaphore::Wait() {
	DWORD dwResult = WaitForSingleObject(_semHandle, INFINITE);
	ASSERT(dwResult == WAIT_OBJECT_0);
}

// --------------------------------------------------------
// Semaphore::WaitAndAllowMessages
// --------------------------------------------------------
void Semaphore::WaitAndAllowMessages() {
	do {
		DWORD dwMsg = MsgWaitForMultipleObjects(
			1, &_semHandle, false, INFINITE, QS_ALLINPUT);
		
		ASSERT(dwMsg != 0xFFFFFFFF);

		switch (dwMsg) {
		case WAIT_OBJECT_0:
			return;
		case WAIT_OBJECT_0+1:
			{
				MSG   msg;
				while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) 
				{ 
						TranslateMessage( &msg ); 
						DispatchMessage( &msg ); 
				}
				
			} break;
		}

	} while(1);
}

// --------------------------------------------------------
// Semaphore::Signal
// --------------------------------------------------------
void Semaphore::Signal() {
	BOOL bResult = ReleaseSemaphore(_semHandle, 1, NULL);
	ASSERT(bResult);
}

// --------------------------------------------------------
// SemaphoreProtection - constructor
// --------------------------------------------------------
SemaphoreProtection::SemaphoreProtection(const Semaphore & sem):
	_semHandle(sem._semHandle) {
	DWORD dwResult = WaitForSingleObject(_semHandle, INFINITE);
	ASSERT(dwResult == WAIT_OBJECT_0);
}

// --------------------------------------------------------
// SemaphoreProtection - destructor
// --------------------------------------------------------
SemaphoreProtection::~SemaphoreProtection() {
	BOOL bResult = ReleaseSemaphore(_semHandle, 1, NULL);
	ASSERT(bResult);
}

// --------------------------------------------------------
// ThreadCompletionAnnouncement - constructor
// --------------------------------------------------------
ThreadCompletionAnnouncement::ThreadCompletionAnnouncement(HANDLE hEvent):
	m_hCompletionEvent(hEvent)
{
	BOOL bResult = ResetEvent(m_hCompletionEvent);
	ASSERT(bResult);
}

// --------------------------------------------------------
// ThreadCompletionAnnouncement - destructor
// --------------------------------------------------------
ThreadCompletionAnnouncement::~ThreadCompletionAnnouncement() {
	BOOL bResult = SetEvent(m_hCompletionEvent);
	ASSERT(bResult);
}

