/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MutexProtection.h,v 1.1.1.1 2003/12/24 19:45:11 gadibergman Exp $
 */

/************************************************************
			MutexProtection.h
			=================

  Synchronization objects, based on Windows synchronization
  functions.
  
************************************************************/

#ifndef __MUTEX_PROTECTION_H
#define __MUTEX_PROTECTION_H

typedef unsigned int uint;

HANDLE CreateNewMutex();

class Mutex {
public:
	Mutex();
	virtual ~Mutex();
	HANDLE _mutexHandle;
};

class MutexProtection {
public:
	MutexProtection(const Mutex & mutex);
	MutexProtection(HANDLE hMutex);
	virtual ~MutexProtection();

private:
	HANDLE _mutexHandle;
};



HANDLE CreateNewSemaphore(uint initialCount, uint maxCount);

class Semaphore {
public:
	Semaphore(uint initialCount, uint maxCount);
	~Semaphore();

	void Wait();
	void WaitAndAllowMessages();
	void Signal();

	HANDLE _semHandle;
};

class SemaphoreProtection {
public:
	SemaphoreProtection(const Semaphore & sem);
	virtual ~SemaphoreProtection();

private:
	HANDLE _semHandle;
};

class ThreadCompletionAnnouncement {
public:
	ThreadCompletionAnnouncement(HANDLE hEvent);

	virtual ~ThreadCompletionAnnouncement();
private:
	HANDLE m_hCompletionEvent;
};


#endif
