/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: ReaderWriter.h,v 1.1.1.1 2003/12/24 19:45:11 gadibergman Exp $
 */

/************************************************************
			ReaderWriter.h
			==============

  Basic reading and writing interfaces using buffer and size.
  
************************************************************/

#ifndef __READER_WRITER_H
#define __READER_WRITER_H


class GeneralReader {
protected:
	virtual unsigned int Read(void * buf, unsigned int numBytes) = 0;
public:
	unsigned int ReadBytes(void * buf, unsigned int numBytes) {
		return Read(buf, numBytes);
	}
};

class GeneralWriter {
private:
	unsigned long m_uBytesWritten;
protected:
	virtual unsigned int Write(const void * buf, unsigned int numBytes) = 0;
public:
	virtual ~GeneralWriter() {}

	GeneralWriter():
		m_uBytesWritten(0)
		{}

	unsigned int WriteBytes(const void * buf, unsigned int numBytes) {
		unsigned int uBytesWritten = Write(buf, numBytes);
		m_uBytesWritten += uBytesWritten;
		return uBytesWritten;
	}

	virtual void DeclareWritingEnded() {}

	unsigned long GetNumBytesWritten() const {
		return m_uBytesWritten;
	}
};



#endif