/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DXSoundReceiver.h,v 1.2 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
			DXSoundReceiver.h
			=================

  An object that attaches to a DirectX sound device,
  receives buffers of raw samples, and redirects them
  to a standard buffer writing object.

************************************************************/

#ifndef __SOUND_RECEIVER_H
#define __SOUND_RECEIVER_H

#include "StreamReceiver.h"

class CDXSoundReceiver : public CSoundReceiverBase {

private:
	void Record();
	void InitNotifications();
	void CreateCaptureBuffer();
	void InitDirectSound();

	unsigned short 
		m_usChannels,
		m_usBytesPerSample,
		m_usSampleRate;

	bool m_bTerminationRequested;

	void Init();

	HANDLE m_hTermination;

	Timestamp m_tsLast;

	// DirectX specific variables
	GUID m_gDeviceID; // Input
	// Internal
	enum { NUM_REC_NOTIFICATIONS = 16 };
	LPDIRECTSOUNDCAPTURE       m_pDSCapture;
	LPDIRECTSOUNDCAPTUREBUFFER m_pDSBCapture;
	LPDIRECTSOUNDNOTIFY        m_pDSNotify;
	DSBPOSITIONNOTIFY          m_aPosNotify[ NUM_REC_NOTIFICATIONS + 1 ];  
	HANDLE                     m_hNotificationEvent; 
	DWORD                      m_dwCaptureBufferSize;
	DWORD                      m_dwNextCaptureOffset;
	DWORD                      m_dwNotifySize;

	void RecordCapturedData();

public:
	void Start();
	void Stop();

	CDXSoundReceiver(
			const GUID * pDeviceID,
			unsigned short usChannels, 
			unsigned short usBytesPerSample,
			unsigned short usSampleRate);

	virtual ~CDXSoundReceiver();

	unsigned short GetChannels() const 
	{ 
		return m_usChannels; 
	}
	
	unsigned short GetBytesPerSample() const 
	{
		return m_usBytesPerSample;
	}

	unsigned short GetSampleRate() const 
	{
		return m_usSampleRate;
	}

	unsigned int GetBlockSize() const 
	{ 
		return m_dwNotifySize; 
	}

	friend /*UINT*/ void CDXSoundReceiver_RecordingThreadFunc(LPVOID pParam);
};

#endif
