/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DirectoriesSelectionPage.cpp,v 1.1.1.1 2003/12/24 19:41:42 gadibergman Exp $
 */

/************************************************************
			DirectoriesSelectionPage.cpp
			============================

  A property page for the selection of a directory
  to store the output files.

************************************************************/

// PCH includes
#include "stdafx.h"

// Project includes
#include "MPEGRecorder.h"
#include "DirectoriesSelectionPage.h"
#include "SpecificApplicationData.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDirectoriesSelectionPage property page

IMPLEMENT_DYNCREATE(CDirectoriesSelectionPage, CPropertyPage)

CDirectoriesSelectionPage::CDirectoriesSelectionPage() : CPropertyPage(CDirectoriesSelectionPage::IDD)
{
	//{{AFX_DATA_INIT(CDirectoriesSelectionPage)
	m_sRecordingPath = _T("");
	//}}AFX_DATA_INIT
}

CDirectoriesSelectionPage::~CDirectoriesSelectionPage()
{
}

void CDirectoriesSelectionPage::SetApplicationData(
	CSpecificApplicationData * pAppData)
{
	m_pAppData = pAppData;
	m_sRecordingPath = pAppData->GetRecordingDirectory();
}

void CDirectoriesSelectionPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDirectoriesSelectionPage)
	DDX_Text(pDX, IDC_RECORDING_PATH, m_sRecordingPath);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDirectoriesSelectionPage, CPropertyPage)
	//{{AFX_MSG_MAP(CDirectoriesSelectionPage)
	ON_BN_CLICKED(IDC_BROWSE_RECORDING_PATH, OnBrowseRecordingPath)
	ON_BN_CLICKED(IDC_EXPLORE_RECORDING_PATH, OnExploreRecordingPath)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDirectoriesSelectionPage message handlers

int CALLBACK BrowseCallbackProc(
    HWND hwnd, 
    UINT uMsg, 
    LPARAM lParam, 
    LPARAM lpData
    )
{
	CDirectoriesSelectionPage * parentDlg = (CDirectoriesSelectionPage*) lpData;

	if (uMsg == BFFM_INITIALIZED) {
		// Set the current folder for an initialized box
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE,
			(LPARAM) (LPCTSTR) parentDlg->m_sRecordingPath);
	}

	return 0;
}

void CDirectoriesSelectionPage::OnBrowseRecordingPath() 
{
	TCHAR dispBuffer[MAX_PATH];

	BROWSEINFO bi = {
		m_hWnd,
		NULL,
		dispBuffer,
		_T("root recording path selection"),
		BIF_RETURNONLYFSDIRS,
		BrowseCallbackProc,
		(LONG) this,
		0
	};

	LPITEMIDLIST pBrowseResult = SHBrowseForFolder(&bi);

	if (!pBrowseResult)
		return;

	TCHAR pathBuffer[MAX_PATH];

	BOOL bResult = SHGetPathFromIDList(pBrowseResult,
		pathBuffer);

	if (bResult) {
		m_sRecordingPath = pathBuffer;
		if (m_sRecordingPath.Right(1) != _T("\\"))
			m_sRecordingPath += _T("\\");
		UpdateData(FALSE);
	}

	CoTaskMemFree(pBrowseResult);
}

void CDirectoriesSelectionPage::OnExploreRecordingPath() 
{
	ShellExecute(m_hWnd, _T("open"), m_sRecordingPath, NULL,
		NULL, SW_SHOW);
}

void CDirectoriesSelectionPage::OnOK() 
{
	UpdateData(TRUE);

	m_pAppData->SetRecordingDirectory(m_sRecordingPath);
	
	CPropertyPage::OnOK();
}
