/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DumbWindowForMessages.cpp,v 1.2 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
			DumbWindowForMessages.cpp
			=========================

  An invisible window that is responsible to receive and
  handle messages from the application system-tray icon.
  It displays the pop-up menu and pops the main UI dialogs
  through the ActiveApplicationObjects object functional 
  layer.

************************************************************/

#include "stdafx.h"
#include "DumbWindowForMessages.h"
#include "ActiveApplicationObjects.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDumbWindowForMessages

CDumbWindowForMessages::CDumbWindowForMessages()
{
	LPCTSTR lpszClassName = 
		AfxRegisterWndClass( 0 ); 

	BOOL bRes = 
		CreateEx( 0, 
		lpszClassName, _T("MPEG Recorder Dummy"), 
		0, 
		0, 0, 0, 0, 
		NULL, NULL );

	ASSERT(bRes);

	m_pObjs = new CActiveApplicationObjects();

	bool bSuccess = m_pObjs->LoadApplicationData();

	if (bSuccess)
		bSuccess = m_pObjs->UseProperties();

	if (!bSuccess) {
		bSuccess = m_pObjs->EditProperties();

		if (!bSuccess) {
			PostQuitMessage( 0 );
			return;
		}

		bSuccess = m_pObjs->UseProperties();

		ASSERT(bSuccess);
	}

	m_uiTimerID = SetTimer(MINUTE_TIMER_EVENT_ID, MINUTE_IN_MS, NULL);
	ASSERT(	m_uiTimerID );
}

CDumbWindowForMessages::~CDumbWindowForMessages()
{
	delete m_pObjs;
}


BEGIN_MESSAGE_MAP(CDumbWindowForMessages, CWnd)
	//{{AFX_MSG_MAP(CDumbWindowForMessages)
	ON_WM_TIMER()
	ON_WM_TIMECHANGE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CDumbWindowForMessages message handlers

LRESULT CDumbWindowForMessages::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if (message == WM_USER)
	{
		UINT uID; 
		UINT uMouseMsg; 
 
		uID = (UINT) wParam;
		ASSERT(uID == 0);

		uMouseMsg = (UINT) lParam; 

		switch(uMouseMsg) {
		case WM_LBUTTONDOWN:
			{
				m_pObjs->WatchVideo(true);
			}

			break;
		case WM_RBUTTONDOWN:
			PopupMenu();

			break;
		};
 
		return 0;
	}
	
	return CWnd::WindowProc(message, wParam, lParam);
}

enum {
	MENU_START = 1001,
};

void Exit(CActiveApplicationObjects& objs, bool& bExit)
{
	bExit = true;
}

void SetProperties(CActiveApplicationObjects& objs, bool& bExit)
{
	if (objs.EditProperties()) {
		bool bSuccess = objs.UseProperties();
		ASSERT(bSuccess);
	}
}

void RecordAudio(CActiveApplicationObjects& objs, bool& bExit)
{
	objs.RecordAudio();
}

void WatchVideo(CActiveApplicationObjects& objs, bool& bExit)
{
	objs.WatchVideo(true);
}

void RecordVideo(CActiveApplicationObjects& objs, bool& bExit)
{
	objs.RecordVideo(false);
}

void ShowRecordingControls(CActiveApplicationObjects& objs, bool& bExit)
{
	ShellExecute	(
						AfxGetMainWnd( )->m_hWnd,
						_T("open"),
						_T("sndvol32.exe"),
						_T("-r -d"),
						NULL,
						SW_SHOWNORMAL
					);
}

void ShowPlaybackControls(CActiveApplicationObjects& objs, bool& bExit)
{
	ShellExecute	(
						AfxGetMainWnd( )->m_hWnd,
						_T("open"),
						_T("sndvol32.exe"),
						_T("-p -d"),
						NULL,
						SW_SHOWNORMAL
					);
}

void ShowMyFiles(CActiveApplicationObjects& objs, bool& bExit)
{
	objs.ShowMyFiles();
}

void ManageRecordingEvents(CActiveApplicationObjects& objs, bool& bExit)
{
	objs.ManageRecordingEvents();
}

void DisplayVolumeMeter(CActiveApplicationObjects& objs, bool& bExit)
{
	objs.DisplayVolumeUnitMeter();
}

struct PopupMenuItem {
	LPCTSTR pszTitle;
	void (*func)(CActiveApplicationObjects& objs, bool &bExit);
};

static PopupMenuItem MenuItems [] = {
	{ _T("Properties"), SetProperties },
	{ _T("My Files Today"), ShowMyFiles },
	{ NULL, NULL }, // separator
	{ _T("Record Audio Now"), RecordAudio },
	{ _T("Watch Video Device"), WatchVideo },
	{ _T("Record Video Now"), RecordVideo },
	{ NULL, NULL }, // separator
	{ _T("Video Recordings Schedule"), ManageRecordingEvents },
	{ NULL, NULL }, // separator
	{ _T("Show Recording Controls"), ShowRecordingControls },
	{ _T("Show Playback Controls"), ShowPlaybackControls },
	{ _T("Display Volume Unit Meter"), DisplayVolumeMeter },
	{ NULL, NULL }, // separator
	{ _T("Exit"), Exit }
};

BOOL CDumbWindowForMessages::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	int iNumMenuItems = sizeof(MenuItems) / sizeof(PopupMenuItem);

	if ((wParam >= MENU_START) && (wParam < MENU_START + iNumMenuItems))
	{
		PopupMenuItem& item = MenuItems[wParam - MENU_START];
		bool bExit(false);
		item.func(*m_pObjs, bExit);
		if (bExit)
		{
			BOOL bRes = KillTimer( m_uiTimerID );
			ASSERT( bRes );
			PostQuitMessage(0);
		}
	}
	
	return CWnd::OnCommand(wParam, lParam);
}

void CDumbWindowForMessages::PopupMenu()
{
	POINT point;
	BOOL bRes;
	bRes = GetCursorPos(&point);
	ASSERT(bRes);
	CMenu menu;
	bRes = menu.CreatePopupMenu();
	ASSERT(bRes);
	bRes = SetForegroundWindow();
	ASSERT(bRes);

	int iNumMenuItems = sizeof(MenuItems) / sizeof(PopupMenuItem);
	for (int i = 0; i < iNumMenuItems; ++i)
	{
		PopupMenuItem& item = MenuItems[i];

		if (item.pszTitle)
			menu.AppendMenu(MF_STRING, i + MENU_START, item.pszTitle);
		else
			menu.AppendMenu(MF_SEPARATOR);
	}
	
	menu.TrackPopupMenu(TPM_CENTERALIGN | TPM_LEFTBUTTON,
		point.x, point.y, this);

}

void CDumbWindowForMessages::OnTimer(UINT nIDEvent) 
{
	m_pObjs->OnMinutePassed();
	
	CWnd::OnTimer(nIDEvent);
}

void CDumbWindowForMessages::OnTimeChange() 
{
	CWnd::OnTimeChange();
	
	m_pObjs->OnTimeChanged();
}
