/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: DummyFilter.cpp,v 1.2 2004/02/20 08:50:34 gadibergman Exp $
 */

/************************************************************
			DummyFilter.cpp
			===============

  A dummy filter that acts as a sink  
************************************************************/

// PCH includes
#include "stdafx.h"

// DX includes
#include "vfwmsgs.h"
#include "mtype.h"

// STL includes
#include <vector>

// Project includes
#include "DummyFilter.h"

#define EMPTY_IMPL(x) \
virtual HRESULT STDMETHODCALLTYPE x { return S_OK; }

// Copied from mtype.cpp because of some link problem
HRESULT WINAPI _CopyMediaType(AM_MEDIA_TYPE *pmtTarget, const AM_MEDIA_TYPE *pmtSource)
{
    //  We'll leak if we copy onto one that already exists - there's one
    //  case we can check like that - copying to itself.
    ASSERT(pmtSource != pmtTarget);
    *pmtTarget = *pmtSource;
    if (pmtSource->cbFormat != 0) {
        ASSERT(pmtSource->pbFormat != NULL);
        pmtTarget->pbFormat = (PBYTE)CoTaskMemAlloc(pmtSource->cbFormat);
        if (pmtTarget->pbFormat == NULL) {
            pmtTarget->cbFormat = 0;
            return E_OUTOFMEMORY;
        } else {
            CopyMemory((PVOID)pmtTarget->pbFormat, (PVOID)pmtSource->pbFormat,
                       pmtTarget->cbFormat);
        }
    }
    if (pmtTarget->pUnk != NULL) {
        pmtTarget->pUnk->AddRef();
    }

    return S_OK;
}

// T derives IUnknown and has uuid
template <class T>
class CSingleInterfaceLocalImpl : public T
{
	int m_cRef;

	HRESULT STDMETHODCALLTYPE QueryInterface( REFIID iid, void ** ppvObject )
	{
		if ( iid == IID_IUnknown )
			*ppvObject = (IUnknown*) this;
		else
		if ( iid == __uuidof(T) )
			*ppvObject = (T*) this;
		else
		{
			*ppvObject = NULL;
			return E_NOINTERFACE;
		}
		++m_cRef;
		return NOERROR;
	}

	ULONG STDMETHODCALLTYPE AddRef()
	{
		return ++m_cRef;
	}

	ULONG STDMETHODCALLTYPE Release()
	{
		if (--m_cRef == 0)
		{
			delete this;
			return 0;
		}
		return m_cRef;
	}

public:
	CSingleInterfaceLocalImpl():
	m_cRef(1)
	{}

};

class CMediaTypesVector : public std::vector<AM_MEDIA_TYPE*>
{
public:
	CMediaTypesVector( IEnumMediaTypes* emt );

	virtual ~CMediaTypesVector();
};

CMediaTypesVector::CMediaTypesVector( IEnumMediaTypes* emt )
{
	if ( FAILED( emt->Reset() ) )
		return;

	AM_MEDIA_TYPE *pmt = NULL;

	while ( S_OK == emt->Next(1, &pmt, NULL) )
		push_back( pmt );			
}

CMediaTypesVector::~CMediaTypesVector()
{
	for (iterator iter = begin(); iter != end(); ++iter)
	{
		AM_MEDIA_TYPE* pmt = *iter;
		DeleteMediaType( pmt );
	}
}

class CSimpleMediaTypesEnumerator : 
public CSingleInterfaceLocalImpl<IEnumMediaTypes>
{
	CMediaTypesVector* m_pvMediaTypes;
	int	m_iPos;
	int m_iSize;

    HRESULT STDMETHODCALLTYPE Next( 
        ULONG cMediaTypes,
        AM_MEDIA_TYPE **ppMediaTypes,
        ULONG *pcFetched);
    
    HRESULT STDMETHODCALLTYPE Skip( ULONG cMediaTypes );
    
    HRESULT STDMETHODCALLTYPE Reset( void );
    
    HRESULT STDMETHODCALLTYPE Clone( IEnumMediaTypes **ppEnum );


public:
	CSimpleMediaTypesEnumerator(CMediaTypesVector* pvMediaTypes);
};

CSimpleMediaTypesEnumerator::CSimpleMediaTypesEnumerator(CMediaTypesVector* pvMediaTypes):
m_pvMediaTypes(pvMediaTypes),
m_iPos(0),
m_iSize(pvMediaTypes->size())
{}

HRESULT STDMETHODCALLTYPE CSimpleMediaTypesEnumerator::Next( 
    ULONG cMediaTypes,
    AM_MEDIA_TYPE **ppMediaTypes,
    ULONG *pcFetched)
{
	if ( !cMediaTypes )
		return S_OK;

	if ( !ppMediaTypes || ( cMediaTypes > 1 ) && !pcFetched )
		return E_POINTER;

	int iMTsLeft = m_iSize - m_iPos;
	bool bHasEnoughToEnum = iMTsLeft >= cMediaTypes;
	int iMTsToGive = ( iMTsLeft > cMediaTypes ) ? cMediaTypes : iMTsLeft;

	for (int i = 0; i < iMTsToGive; ++i, ++m_iPos)
	{
		AM_MEDIA_TYPE * pmtSrc = (*m_pvMediaTypes)[m_iPos];
		AM_MEDIA_TYPE * pmtTgt = CreateMediaType( pmtSrc );
		ppMediaTypes[i] = pmtTgt;
	}

	if ( pcFetched )
		*pcFetched = iMTsToGive;

	return bHasEnoughToEnum ? S_OK : S_FALSE;
}

HRESULT STDMETHODCALLTYPE CSimpleMediaTypesEnumerator::Skip( ULONG cMediaTypes )
{
	m_iPos += cMediaTypes;
	if ( m_iPos >= m_iSize )
		return S_FALSE;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CSimpleMediaTypesEnumerator::Reset( void )
{
	m_iPos = 0;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CSimpleMediaTypesEnumerator::Clone( IEnumMediaTypes **ppEnum )
{
	if ( !ppEnum )
		return E_POINTER;

	CSimpleMediaTypesEnumerator* pCopy =
		new CSimpleMediaTypesEnumerator(m_pvMediaTypes);

	pCopy->m_iPos = m_iPos;

	return S_OK;
}

class CDummyInputPin : 
public CSingleInterfaceLocalImpl<IPin>
{
	IPin* m_pConnector;
	AM_MEDIA_TYPE* m_pMediaType;
	CMediaTypesVector* m_pvMediaTypes;
	IBaseFilter* m_pFilter;

    HRESULT STDMETHODCALLTYPE Connect( 
        IPin *pReceivePin,
        const AM_MEDIA_TYPE *pmt);
    
    HRESULT STDMETHODCALLTYPE ReceiveConnection( 
        IPin *pConnector,
        const AM_MEDIA_TYPE *pmt);
    
    HRESULT STDMETHODCALLTYPE Disconnect( void );
    
    HRESULT STDMETHODCALLTYPE ConnectedTo( IPin **pPin );
    
    HRESULT STDMETHODCALLTYPE ConnectionMediaType( AM_MEDIA_TYPE *pmt );
    
    HRESULT STDMETHODCALLTYPE QueryPinInfo( PIN_INFO *pInfo );
    
    HRESULT STDMETHODCALLTYPE QueryDirection( PIN_DIRECTION *pPinDir );
    
    HRESULT STDMETHODCALLTYPE QueryId( LPWSTR *Id );
    
	EMPTY_IMPL( QueryAccept( const AM_MEDIA_TYPE *pmt ) )
    
    HRESULT STDMETHODCALLTYPE EnumMediaTypes( IEnumMediaTypes **ppEnum );
    
    HRESULT STDMETHODCALLTYPE QueryInternalConnections( 
        IPin **apPin,
        ULONG *nPin)
	{
		return E_NOTIMPL;
	}
    
    EMPTY_IMPL( EndOfStream( void ) )
    
    EMPTY_IMPL( BeginFlush( void ) )
    
    EMPTY_IMPL( EndFlush( void ) )
    
    EMPTY_IMPL( NewSegment( 
		REFERENCE_TIME tStart,
        REFERENCE_TIME tStop,
        double dRate) )
  
public:
	CDummyInputPin(IBaseFilter* pFilter, IPin* pOutPin);

	virtual ~CDummyInputPin();

};

CDummyInputPin::CDummyInputPin(IBaseFilter* pFilter, IPin* pOutPin):
m_pConnector(NULL),
m_pMediaType(NULL),
m_pvMediaTypes(NULL),
m_pFilter(pFilter)
{
	IEnumMediaTypes *pMediaTypesEnum;
	pOutPin->EnumMediaTypes( &pMediaTypesEnum );

	m_pvMediaTypes = new CMediaTypesVector( pMediaTypesEnum );
}

CDummyInputPin::~CDummyInputPin()
{
	if ( m_pConnector )
		m_pConnector->Release();

	if ( m_pMediaType )
		DeleteMediaType( m_pMediaType );

	if ( m_pvMediaTypes )
		delete m_pvMediaTypes;
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::Connect( 
    IPin *pReceivePin,
    const AM_MEDIA_TYPE *pmt)
{
	ATLASSERT(false); // This is an input pin!
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::ReceiveConnection( 
    IPin *pConnector,
    const AM_MEDIA_TYPE *pmt)
{
	if ( m_pConnector )	
		return VFW_E_ALREADY_CONNECTED;

	pConnector->AddRef();
	m_pConnector = pConnector;

	if ( pmt )
		m_pMediaType = CreateMediaType( pmt );

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::Disconnect( void )
{
	if ( m_pConnector )
	{
		m_pConnector->Release();
		m_pConnector = NULL;

		if ( m_pMediaType )
		{
			DeleteMediaType( m_pMediaType );
			m_pMediaType = NULL;
		}

		return S_OK;
	}
	return S_FALSE;
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::ConnectedTo( IPin **pPin )
{
	if ( !pPin )
		return E_POINTER;
	if ( !m_pConnector )
		return VFW_E_NOT_CONNECTED;
	m_pConnector->AddRef();
	*pPin = m_pConnector;
	return S_OK;
}
    
HRESULT STDMETHODCALLTYPE CDummyInputPin::ConnectionMediaType( AM_MEDIA_TYPE *pmt )
{
	if ( !m_pMediaType )
		return VFW_E_NOT_CONNECTED;

	if ( !pmt )
		return E_POINTER;

	return _CopyMediaType( pmt, m_pMediaType );
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::QueryPinInfo( PIN_INFO *pInfo )
{
	if ( !pInfo )
		return E_POINTER;

	pInfo->pFilter = m_pFilter;
	m_pFilter->AddRef();
	pInfo->dir = PINDIR_INPUT;
	wcscpy( pInfo->achName, L"Dummy input pin");

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::QueryDirection( PIN_DIRECTION *pPinDir )
{
	if ( !pPinDir )
		return E_POINTER;

	*pPinDir = PINDIR_INPUT;
	return S_OK;
}

#define DUMMY_SINK_PIN_ID L"Dummy Input Pin ID"

HRESULT STDMETHODCALLTYPE CDummyInputPin::QueryId( LPWSTR *Id )
{
	static LPCWSTR pwszPinID = DUMMY_SINK_PIN_ID;
	static int iPinIDLength = wcslen( pwszPinID );

	if ( !Id )
		return E_POINTER;

	*Id = (LPWSTR) CoTaskMemAlloc( ( iPinIDLength + 1 ) * sizeof(WCHAR) );
	if ( !Id )
		return E_OUTOFMEMORY;

	wcscpy( *Id, pwszPinID );
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummyInputPin::EnumMediaTypes( IEnumMediaTypes **ppEnum )
{
	if ( !ppEnum )
		return E_POINTER;

	*ppEnum = new CSimpleMediaTypesEnumerator( m_pvMediaTypes );
	return S_OK;
}

class CSinglePinEnumerator : 
public CSingleInterfaceLocalImpl<IEnumPins>
{
private:
	bool m_bDone;
	IPin* m_pPin;

public:

	CSinglePinEnumerator(IPin* pPin);

	virtual ~CSinglePinEnumerator();
	
private:

    HRESULT STDMETHODCALLTYPE Next( 
        ULONG cPins,
        IPin **ppPins,
        ULONG *pcFetched);
    
    HRESULT STDMETHODCALLTYPE Skip( ULONG cPins );
    
    HRESULT STDMETHODCALLTYPE Reset(void);
    
    HRESULT STDMETHODCALLTYPE Clone( IEnumPins **ppEnum );

};

CSinglePinEnumerator::CSinglePinEnumerator(IPin* pPin):
m_pPin(pPin),
m_bDone(false)
{
	m_pPin->AddRef();
}

CSinglePinEnumerator::~CSinglePinEnumerator()
{
	m_pPin->Release();
}

HRESULT STDMETHODCALLTYPE CSinglePinEnumerator::Next( 
    ULONG cPins,
    IPin **ppPins,
    ULONG *pcFetched)
{
	if ( !cPins )
		return S_OK;

	if ( m_bDone )
	{
		if ( pcFetched )
			*pcFetched = 0;
		return S_FALSE;
	}

	m_pPin->AddRef();
	ppPins[0] = m_pPin;

	m_bDone = true;

	if ( cPins > 1 )
	{
		if ( !pcFetched )
			return E_POINTER;
		*pcFetched = 1;
		return S_FALSE;
	}

	if ( pcFetched )
		*pcFetched = 1;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CSinglePinEnumerator::Skip( 
    ULONG cPins)
{
	if ( cPins )
		m_bDone = true;

	if ( m_bDone )
		return S_FALSE;

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CSinglePinEnumerator::Reset(void)
{
	m_bDone = false;
	return S_OK;
}
    
HRESULT STDMETHODCALLTYPE CSinglePinEnumerator::Clone( IEnumPins **ppEnum )
{
	if ( !ppEnum )
		return E_POINTER;

	CSinglePinEnumerator* pOther = new CSinglePinEnumerator(m_pPin);
	pOther->m_bDone = m_bDone;
	*ppEnum = pOther;

	return S_OK;
}

// {50B93EFB-E57E-4d78-B2F7-7D2D45963A29}
static const GUID CLSID_DummySinkFilter = 
{ 0x50b93efb, 0xe57e, 0x4d78, { 0xb2, 0xf7, 0x7d, 0x2d, 0x45, 0x96, 0x3a, 0x29 } };


HRESULT STDMETHODCALLTYPE CDummySinkFilter::Stop()
{ 
	m_fsFilterState = State_Stopped; 
	return S_OK; 
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::Pause()
{
	m_fsFilterState = State_Paused;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::Run(REFERENCE_TIME)
{
	m_fsFilterState = State_Running;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::GetState(DWORD, FILTER_STATE *m_pfsFilterState)
{
	*m_pfsFilterState = m_fsFilterState;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::SetSyncSource(IReferenceClock *pClock)
{
	m_pRefClock = pClock;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::GetSyncSource(IReferenceClock **pClock)
{
	*pClock = m_pRefClock;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::EnumPins( IEnumPins **ppEnum )
{
	if ( !ppEnum )
		return E_POINTER;

	CSinglePinEnumerator* pEnum = new CSinglePinEnumerator(m_pSinkInputPin);

	if ( !pEnum )
		return E_OUTOFMEMORY;

	*ppEnum = pEnum;
	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::FindPin( 
    LPCWSTR Id,
    IPin **ppPin)
{
	if ( !ppPin )
		return E_POINTER;

	if ( !Id )
	{
		*ppPin = NULL;
		return E_POINTER;
	}

	if ( 0 == wcsicmp( Id, DUMMY_SINK_PIN_ID ) )
	{
		*ppPin = m_pSinkInputPin;
		m_pSinkInputPin->AddRef();
		return S_OK;
	}
	else
	{	
		*ppPin = NULL;
		return VFW_E_NOT_FOUND;
	}
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::QueryFilterInfo( FILTER_INFO *pInfo )
{
	if ( !pInfo )
		return E_POINTER;

	wcscpy( pInfo->achName, m_fiFilterInfo.achName );
	pInfo->pGraph = m_fiFilterInfo.pGraph;

	if ( m_fiFilterInfo.pGraph )
		m_fiFilterInfo.pGraph->AddRef();

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::JoinFilterGraph( 
    IFilterGraph *pGraph,
    LPCWSTR pName)
{
	if ( m_fiFilterInfo.pGraph )
		m_fiFilterInfo.pGraph->Release();

	wcscpy( m_fiFilterInfo.achName, pName );
	m_fiFilterInfo.pGraph = pGraph;

//	if ( pGraph )
//		pGraph->AddRef();

	return S_OK;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::QueryVendorInfo( LPWSTR *pVendorInfo )
{
	return E_NOTIMPL;
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::GetClassID( CLSID *pClassID )
{
	if ( !pClassID )
		return E_POINTER;

	*pClassID = CLSID_DummySinkFilter;
	return S_OK;
}

CDummySinkFilter::CDummySinkFilter(IPin* pOutputPin):
m_pSinkInputPin(new CDummyInputPin(this, pOutputPin))
{
	m_fiFilterInfo.pGraph = NULL;

	HRESULT hRes = CoRegisterClassObject( CLSID_DummySinkFilter, this,
		CLSCTX_INPROC_SERVER, REGCLS_SINGLEUSE, &m_dwClassRegistration );

	ATLASSERT(hRes == S_OK);
}

CDummySinkFilter::~CDummySinkFilter()
{
	m_pSinkInputPin->Release();

//	if ( m_fiFilterInfo.pGraph )
//		m_fiFilterInfo.pGraph->Release();

	CoRevokeClassObject( m_dwClassRegistration );
}

HRESULT STDMETHODCALLTYPE CDummySinkFilter::QueryInterface( REFIID iid, void ** ppvObject )
{
	if ( iid == IID_IUnknown )
		*ppvObject = (IUnknown*) this;
	else
	if ( iid == IID_IBaseFilter )
		*ppvObject = (IBaseFilter*) this;
	else
	if ( iid == IID_IMediaFilter )
		*ppvObject = (IMediaFilter*) this;
	else
	if ( iid == IID_IPersist )
		*ppvObject = (IPersist*) this;
	else
	{
		*ppvObject = NULL;
		return E_NOINTERFACE;
	}
	return NOERROR;
}

ULONG STDMETHODCALLTYPE CDummySinkFilter::AddRef()
{
	return 2;
}

ULONG STDMETHODCALLTYPE CDummySinkFilter::Release()
{
	return 1;
}

static void TestPin()
{
	new CDummySinkFilter(NULL);
}