/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: EventScheduleReport.cpp,v 1.1.1.1 2003/12/24 19:41:38 gadibergman Exp $
 */

/************************************************************
			EventScheduledReport.cpp
			========================

  User interface that provides information about the 
  scheduled recording events chronologically, as detailed
  as recording starting and ending time points.

************************************************************/

#include "stdafx.h"
#include "EventScheduleReport.h"
#include "RepeatingTimeEvent.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEventScheduleReport dialog


CEventScheduleReport::CEventScheduleReport(CWnd* pParent /*=NULL*/):
CDialog(CEventScheduleReport::IDD, pParent),
m_pvTimeEvents(NULL)
{
	//{{AFX_DATA_INIT(CEventScheduleReport)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CEventScheduleReport::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEventScheduleReport)
	DDX_Control(pDX, IDC_EVENTS_LIST, m_EventsList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEventScheduleReport, CDialog)
	//{{AFX_MSG_MAP(CEventScheduleReport)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEventScheduleReport message handlers

BOOL CEventScheduleReport::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	CreateReportColumns();
	UpdateListUI();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CEventScheduleReport::ShowTimeSortedEvents(const CTimeEventsVector *pvTimeEvents)
{
	m_pvTimeEvents = pvTimeEvents;
	DoModal();
	m_pvTimeEvents = NULL;
}

enum {
	DATE_COLUMN,
	TIME_COLUMN,
	ACTION_COLUMN,

	NUMBER_OF_COLUMNS
};

enum {
	MAX_LIST_ITEMS = 20
};

struct ColumnInfo {
	TCHAR* m_pszColText;
	int m_iNumPixels;
};

static ColumnInfo g_vColumnInfo[NUMBER_OF_COLUMNS] = {
	{ _T("Date"), 80 },
	{ _T("Time"), 80 },
	{ _T("Action"), 150 }
};

void CEventScheduleReport::UpdateListUI()
{
	m_EventsList.DeleteAllItems( );
	
	CSimpleTimePoint tpStart;
	tpStart.SetLocalTime();

	for (int iIndex(0); iIndex < MAX_LIST_ITEMS; ++iIndex)
	{
		CSimpleTimePoint tpEarliest;
		CRepeatingTimeEvent tEvent;
		bool bIsEnding;

		bool bSuccess = 
			m_pvTimeEvents->GetEarliestStateChange(
			tpStart, tpEarliest, bIsEnding, tEvent);

		if (!bSuccess)
			break;

		std::string strDate, strTime;
		tpEarliest.m_Date.GetString(strDate);
		tpEarliest.m_Time.GetString(strTime);

		int iInsertedIndex = m_EventsList.InsertItem( iIndex, _T("") );
		ASSERT(iInsertedIndex == iIndex);

		BOOL bRes;
		bRes = m_EventsList.SetItemText( 
			iIndex, DATE_COLUMN, strDate.c_str() );
		ASSERT(bRes);
		bRes = m_EventsList.SetItemText( 
			iIndex, TIME_COLUMN, strTime.c_str() );
		ASSERT(bRes);

		LPCTSTR pszAction = bIsEnding ?
			_T("Stop Recording") : _T("Start Recording");

		bRes = m_EventsList.SetItemText( 
			iIndex, ACTION_COLUMN, pszAction );
		ASSERT(bRes);

		tpStart = tpEarliest;
	}

}

void CEventScheduleReport::CreateReportColumns()
{
	for (int i = 0; i < NUMBER_OF_COLUMNS; ++i)
	{
		m_EventsList.InsertColumn(
			i, g_vColumnInfo[i].m_pszColText, 
			LVCFMT_LEFT, g_vColumnInfo[i].m_iNumPixels,
			i	);
	}
}
