/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: HardwarePropertiesPage.cpp,v 1.2 2004/01/25 17:45:33 gadibergman Exp $
 */

/************************************************************
			HardwarePropertiesPage.cpp
			==========================

  A UI properties page that allows the user to select
  a video input device, an audio input device,
  and a local TV format.

************************************************************/

// PCH includes
#include "stdafx.h"

// DirectX includes
#include "strmif.h"

// Project includes
#include "MPEGRecorder.h"
#include "HardwarePropertiesPage.h"
#include "VideoCaptureDevices.h"
#include "SoundCaptureDevices.h"
#include "SpecificApplicationData.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHardwarePropertiesPage property page

IMPLEMENT_DYNCREATE(CHardwarePropertiesPage, CPropertyPage)

CHardwarePropertiesPage::CHardwarePropertiesPage() : 
CPropertyPage(CHardwarePropertiesPage::IDD),
m_pAppData(NULL),
m_pSoundDevices(new CSoundCaptureDevices),
m_pVideoDevices(new CVideoCaptureDevices)
{
	//{{AFX_DATA_INIT(CHardwarePropertiesPage)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void CHardwarePropertiesPage::SetApplicationData(CSpecificApplicationData * pAppData) {
	m_pAppData = pAppData;
}

CHardwarePropertiesPage::~CHardwarePropertiesPage()
{
	delete m_pSoundDevices;
	delete m_pVideoDevices;
}

void CHardwarePropertiesPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHardwarePropertiesPage)
	DDX_Control(pDX, IDC_PREF_IMAGE_SIZE_COMBO, m_PreferredImageSizeCombo);
	DDX_Control(pDX, IDC_TV_VIDEO_FORMATS_COMBO, m_TVVideoFormatsCombo);
	DDX_Control(pDX, IDC_VIDEO_CAPTURE_DEVICES_COMBO, m_VideoDevicesCombo);
	DDX_Control(pDX, IDC_RECORDING_DEVICE_COMBO, m_AudioDevicesCombo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CHardwarePropertiesPage, CPropertyPage)
	//{{AFX_MSG_MAP(CHardwarePropertiesPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHardwarePropertiesPage message handlers

void CHardwarePropertiesPage::ShowAudioDevices()
{
	CString 
		sSelectedAudioDeviceName = m_pAppData->GetRecordingDeviceName();

	const CSoundCaptureDevices::DevicesVector vDevices =
		m_pSoundDevices->GetDevices();

	if (vDevices.size()) {

		CSoundCaptureDevices::DevicesVector::const_iterator iter;
		bool bSelectedDevice = false;

		for (iter = vDevices.begin(); iter != vDevices.end(); ++iter) {
			const RecordingDevice & dev = *iter;

			// Add the string to the combo box
			int devIndex = m_AudioDevicesCombo.AddString(dev.m_sDesc);

			if (dev.m_sDesc == sSelectedAudioDeviceName) {
				m_AudioDevicesCombo.SetCurSel(devIndex);
				bSelectedDevice = true;
			}

		}

		// Select the first device in the combo box
		if (!bSelectedDevice)
			m_AudioDevicesCombo.SetCurSel(0);
	}
}

void CHardwarePropertiesPage::ShowVideoDevices()
{
	CString
		sSelectedVideoDeviceName = m_pAppData->GetVideoCaptureDevName();

	const CVideoCaptureDevices::CDevicesList vVideoDevices = 
		m_pVideoDevices->GetDevices();

	if (vVideoDevices.size()) {
		CVideoCaptureDevices::CDevicesList::const_iterator iter;
		bool bSelectedDevice = false;

		for (iter = vVideoDevices.begin(); iter != vVideoDevices.end(); ++iter) {
			const CVidCapDevDesc & devDesc = *iter;

			const CString & devName = devDesc.GetDeviceName();
			// Add the string to the combo box
			int devIndex = m_VideoDevicesCombo.AddString(devName);

			if (devName == sSelectedVideoDeviceName) {
				m_VideoDevicesCombo.SetCurSel(devIndex);
				bSelectedDevice = true;
			}
		}

		// Select the first device in the combo box
		if (!bSelectedDevice)
			m_VideoDevicesCombo.SetCurSel(0);
	}
}

BOOL CHardwarePropertiesPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();
	
	ShowAudioDevices();
	ShowVideoDevices();
	ShowPreferredImageSizes();
	ShowTVVideoFormats();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CHardwarePropertiesPage::OnOK() 
{
	UpdateData(TRUE);

	int nAudioCaptureIndex = m_AudioDevicesCombo.GetCurSel();

	if (nAudioCaptureIndex < 0) {
		AfxMessageBox("No audio device selected");
		return;
	}

	int nVideoCaptureIndex = m_VideoDevicesCombo.GetCurSel();

	if (nVideoCaptureIndex < 0) {
		AfxMessageBox("No video device selected");
		return;
	}

	int nTVVideoFormatsIndex = m_TVVideoFormatsCombo.GetCurSel();

	if (nTVVideoFormatsIndex < 0) {
		AfxMessageBox("No TV video format selected");
		return;
	}

	int iPreferredImageSizeIndex = m_PreferredImageSizeCombo.GetCurSel();

	if ( iPreferredImageSizeIndex < 0 ) {
		AfxMessageBox("No preferred image size selected");
		return;
	}

	CString
		sSelectedAudioDeviceName,
		sSelectedVideoDeviceName,
		sPreferredImageSize;

	m_AudioDevicesCombo.GetLBText(nAudioCaptureIndex, sSelectedAudioDeviceName);
	m_VideoDevicesCombo.GetLBText(nVideoCaptureIndex, sSelectedVideoDeviceName);
	m_PreferredImageSizeCombo.GetLBText(iPreferredImageSizeIndex, sPreferredImageSize);

	m_pAppData->SetRecordingDeviceName(sSelectedAudioDeviceName);
	m_pAppData->SetVideoCaptureDevName(sSelectedVideoDeviceName);
	m_pAppData->SetPreferredImageSize(sPreferredImageSize);
	m_pAppData->SetTVVideoFormatNumber(m_TVVideoFormatsCombo.GetItemData(nTVVideoFormatsIndex));

	CPropertyPage::OnOK();
}

struct TVVideoFormatInfo {
	long lFormatNumber;
	const char * formatName;
};

void CHardwarePropertiesPage::ShowTVVideoFormats()
{
	static TVVideoFormatInfo formats[] = {
        { AnalogVideo_None,		"None" },
        { AnalogVideo_NTSC_M,	"NTSC M" },
        { AnalogVideo_NTSC_M_J,	"NTSC M J" },
        { AnalogVideo_NTSC_433, "NTSC 433" },
        { AnalogVideo_PAL_B,	"PAL B" },
        { AnalogVideo_PAL_D,	"PAL D" },
        { AnalogVideo_PAL_H,	"PAL H" },
        { AnalogVideo_PAL_I,	"PAL I" },
        { AnalogVideo_PAL_M,	"PAL M" },
        { AnalogVideo_PAL_N,	"PAL N" },
        { AnalogVideo_PAL_60,	"PAL 60" },
        { AnalogVideo_SECAM_B,	"SECAM B" },
        { AnalogVideo_SECAM_D,  "SECAM D" },
        { AnalogVideo_SECAM_G,	"SECAM G" },
        { AnalogVideo_SECAM_H,	"SECAM H" },
        { AnalogVideo_SECAM_K,	"SECAM K" },
        { AnalogVideo_SECAM_K1,	"SECAM K1" },
        { AnalogVideo_SECAM_L,	"SECAM L" },
        { AnalogVideo_SECAM_L1,	"SECAM L1" },
        { AnalogVideo_PAL_N_COMBO, "PAL N COMBO" }
    };

	DWORD uTVVideoFormatNumber = m_pAppData->GetTVVideoFormatNumber();

	bool bSelectedFormat = false;

	for (int i = 0; i < sizeof(formats)/sizeof(TVVideoFormatInfo); ++i) {
		int devIndex = m_TVVideoFormatsCombo.AddString(formats[i].formatName);
		m_TVVideoFormatsCombo.SetItemData(devIndex, formats[i].lFormatNumber);

		if (formats[i].lFormatNumber == uTVVideoFormatNumber) {
			bSelectedFormat = true;
			m_TVVideoFormatsCombo.SetCurSel(devIndex);
		}
	}

	if (!bSelectedFormat)
		m_TVVideoFormatsCombo.SetCurSel(0);
}

void CHardwarePropertiesPage::ShowPreferredImageSizes()
{
	static LPCSTR vpszPrefImageSizes[] = {
		DEFAULT_PREF_IMAGE_SIZE,
		BEST_PREF_IMAGE_SIZE,
		"320x240",
		"352x288",
		"640x480",
		"720x576"
	};

	const CString & cstrPrefImageSize = m_pAppData->GetPreferredImageSize();

	bool bFound(false);
	
	int iPrefImageSizes = sizeof(vpszPrefImageSizes) / sizeof(LPCSTR);

	for ( int i = 0; i < iPrefImageSizes; ++i )
	{
		int ipisIndex = m_PreferredImageSizeCombo.AddString( vpszPrefImageSizes[i] );
		if ( 0 == _tcsicmp( cstrPrefImageSize, vpszPrefImageSizes[i] ) )
		{
			bFound = true;
			m_PreferredImageSizeCombo.SetCurSel( ipisIndex );
		}	
	}

	if ( !bFound )
		m_PreferredImageSizeCombo.SetCurSel( 0 );
}
