// ImageCaptureDlg.cpp : implementation file
//

// PCH includes
#include "stdafx.h"

// Project includes
#include "DXVideoReceiver.h"
#include "ImageCaptureDlg.h"
#include "MediaDivision.h"
#include "TimeKeepingReaderWriter.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CImageCaptureDlg dialog


CImageCaptureDlg::CImageCaptureDlg(CWnd* pParent)
	: CDialog(CImageCaptureDlg::IDD, pParent),
	m_pParent(pParent)
{
	//{{AFX_DATA_INIT(CImageCaptureDlg)
	m_bHasDelay = FALSE;
	m_uiDelaySeconds = 0;
	//}}AFX_DATA_INIT

	m_uiDelaySeconds = 5;
}


void CImageCaptureDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CImageCaptureDlg)
	DDX_Check(pDX, IDC_HAVE_CAPTURE_DELAY, m_bHasDelay);
	DDX_Text(pDX, IDC_SECS_BEFORE_CAPTURE, m_uiDelaySeconds);
	DDV_MinMaxUInt(pDX, m_uiDelaySeconds, 0, 60);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CImageCaptureDlg, CDialog)
	//{{AFX_MSG_MAP(CImageCaptureDlg)
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CImageCaptureDlg message handlers

class CImageCaptureWriter : public TimeKeepingWriter
{
private:
	CDXVideoReceiver* m_pVideoReceiver;
	CMediaProvider* m_pMediaProvider;
	bool m_bImageCaptured;
	HANDLE m_hCompletionEvent;
	void * m_pBuffer;
	long m_lBufferSize;

	unsigned int Write(
			const void * buf, 
			unsigned int numBytes,
			Timestamp timestamp,
			Duration duration
			);

	void DeclareWritingEnded();

	// Forbidden methods
	CImageCaptureWriter(const CImageCaptureWriter&);
	CImageCaptureWriter& operator=(const CImageCaptureWriter&);

public:

	CImageCaptureWriter(
		CDXVideoReceiver* pVideoReceiver
			);

	void WaitForCapture();

	bool IsImageCaptured() const { return m_bImageCaptured; }

	void GetCaptureBuffer( void** ppBuffer, long* plBufferSize )
	{
		*ppBuffer = m_pBuffer;
		*plBufferSize = m_lBufferSize;
	}

	virtual ~CImageCaptureWriter();
};

CImageCaptureWriter::CImageCaptureWriter
(
	CDXVideoReceiver* pVideoReceiver
):
m_pVideoReceiver(pVideoReceiver),
m_bImageCaptured(false),
m_hCompletionEvent(CreateEvent(NULL, TRUE, FALSE, NULL)),
m_pBuffer(NULL),
m_lBufferSize(0)
{
	m_pMediaProvider = pVideoReceiver->CreateMediaProvider(this);
}

void WriteBMPFile(CWnd* pWndOwner,
				  const void* pBuffer, 
				  long lBufferSize,
				  long lWidth, 
				  long lHeight
				 );

unsigned int CImageCaptureWriter::Write(
		const void * buf, 
		unsigned int numBytes,
		Timestamp timestamp,
		Duration duration
		)
{
	if ( m_bImageCaptured )
		return 0;

	m_bImageCaptured = true;
	m_pMediaProvider->Stop();

	m_pBuffer = malloc(numBytes);
	memcpy( m_pBuffer, buf, numBytes );
	m_lBufferSize = numBytes;

	return numBytes;
}

void CImageCaptureWriter::DeclareWritingEnded()
{
	BOOL bRes = SetEvent( m_hCompletionEvent );
	ASSERT( bRes );
}

void CImageCaptureWriter::WaitForCapture()
{
	do 
	{
		DWORD dwMsg = MsgWaitForMultipleObjects(
			1, 
			&m_hCompletionEvent, 
			TRUE, 
			INFINITE, 
			QS_ALLINPUT
		);

		ASSERT(dwMsg != 0xFFFFFFFF);

		switch (dwMsg) {
		case WAIT_OBJECT_0:
			return;

		case WAIT_OBJECT_0+1:
			{
				MSG   msg;
				while( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) 
				{ 
						TranslateMessage( &msg ); 
						DispatchMessage( &msg ); 
				}
				
			} break;
		}

	}
	while(1);
}

CImageCaptureWriter::~CImageCaptureWriter()
{
	WaitForCapture();

	free(m_pBuffer);
}

void CImageCaptureDlg::CaptureImage(CDXVideoReceiver *pVideoReceiver)
{
	m_pVideoReceiver = pVideoReceiver;

	DoModal();
}

void CImageCaptureDlg::OnOK() 
{
	UpdateData( TRUE );

	if ( m_bHasDelay && m_uiDelaySeconds )
	{
		m_uiDelayTimerID = SetTimer( DELAY_TIMER_ID, 
			1000, NULL );
	}
	else
	{
		CaptureImageNow();
	}
}

void CImageCaptureDlg::OnTimer(UINT nIDEvent) 
{
	if ( nIDEvent == DELAY_TIMER_ID )
	{
		--m_uiDelaySeconds;
		UpdateData(FALSE);

		if ( !m_uiDelaySeconds )
		{
			BOOL bRes = KillTimer( m_uiDelayTimerID );
			ASSERT( bRes );

			CaptureImageNow();
		}
	}
	
	CDialog::OnTimer(nIDEvent);
}

void CImageCaptureDlg::CaptureImageNow()
{
	CImageCaptureWriter icwWriter(m_pVideoReceiver);

	icwWriter.WaitForCapture();

	if ( icwWriter.IsImageCaptured() )
	{
		void * pBuffer;
		long lBufferSize;

		icwWriter.GetCaptureBuffer(&pBuffer, &lBufferSize);

		WriteBMPFile(
					m_pParent, 
					pBuffer, 
					lBufferSize,
					m_pVideoReceiver->GetWidth(),
					m_pVideoReceiver->GetHeight()
					);
	}

	CDialog::OnOK();
}

void WriteBMPFile(CWnd* pWndOwner,
				  const void* pBuffer, 
				  long lBufferSize,
				  long lWidth, 
				  long lHeight
				 )
{
	static char BASED_CODE szFilter[] = "Bitmap Files (*.bmp) |*.bmp| All Files (*.*) |*.*||";
	CFileDialog ofdFileDialog(FALSE,"bmp",NULL,OFN_LONGNAMES,szFilter,pWndOwner);

	if (ofdFileDialog.DoModal() != IDOK)
		return;

    // write out a BMP file

	LPCTSTR szFileName = ofdFileDialog.m_ofn.lpstrFile;

    HANDLE hf = CreateFile(
        szFileName, GENERIC_WRITE, 0, NULL,
        CREATE_ALWAYS, NULL, NULL );

    if( hf == INVALID_HANDLE_VALUE )
        return;

    // write out the file header
    //
    BITMAPFILEHEADER bfh;
    memset( &bfh, 0, sizeof( bfh ) );
    bfh.bfType = 'MB';
    bfh.bfSize = sizeof( bfh ) + lBufferSize + sizeof( BITMAPINFOHEADER );
    bfh.bfOffBits = sizeof( BITMAPINFOHEADER ) + sizeof( BITMAPFILEHEADER );

    DWORD dwWritten = 0;
    WriteFile( hf, &bfh, sizeof( bfh ), &dwWritten, NULL );

    // and the bitmap format
    //
    BITMAPINFOHEADER bih;
    memset( &bih, 0, sizeof( bih ) );
    bih.biSize = sizeof( bih );
    bih.biWidth = lWidth;
    bih.biHeight = lHeight;
    bih.biPlanes = 1;
    bih.biBitCount = 24;

    dwWritten = 0;
    WriteFile( hf, &bih, sizeof( bih ), &dwWritten, NULL );

    // and the bits themselves
    //
    dwWritten = 0;
    WriteFile( hf, pBuffer, lBufferSize, &dwWritten, NULL );
    CloseHandle( hf );
}
