/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LameEncoder.h,v 1.1.1.1 2003/12/24 19:41:36 gadibergman Exp $
 */

/************************************************************
			LameEncoder.h
			=============

  An MP3 audio encoder that receives standard raw input buffers
  and writes standard encoded output buffers.
  Uses code from the Lame project.

************************************************************/

#ifndef __LAME_ENCODER_H
#define __LAME_ENCODER_H

// Includes
#include "lame.h"
#include "AudioEncoder.h"

#include "mpeg4/combined/media_time.h"

class TimeKeepingReader;
class TimeKeepingWriter;

class CLameEncoder : public CAudioEncoder {

private:
	int GetUnpackedSamples(int samples_to_read, int bytes_per_sample, int swap_order, int *sample_buffer);
	unsigned int m_uSamplesRead;
	int ReadPCMSamples(int sample_buffer[2304], int frame_size, int samples_to_read);
	int GetAudio(int Buffer[2][1152]);
	void Encode();
	void Init();
	lame_global_flags * m_pGlobalFlags;
	TimeKeepingReader * m_pReader;
	TimeKeepingWriter * m_pWriter;
	Timestamp m_timestamp;
	Duration m_duration;

	unsigned short 
		m_usChannels,
		m_usBytesPerSample,
		m_usSampleRate,
		m_usKBitRate;

	HANDLE m_hTermination;

public:

	CLameEncoder(
					TimeKeepingReader * pReader, 
					TimeKeepingWriter * pWriter,
					unsigned short usChannels, 
					unsigned short usBytesPerSample,
					unsigned short usSampleRate,
					unsigned short usKBitRate
				);

	virtual ~CLameEncoder();
	
	friend void/*UINT */CLameEncoder_EncodingThreadFunc(LPVOID pParam);

	MediaType get_audio_mp4_fileinfo(
					 bool *mpeg4,
					 bool *isma_compliant,
					 uint8_t *audioProfile,
					 uint8_t **audioConfig,
					 uint32_t *audioConfigLen,
					 uint8_t *mp4AudioType);
};

#endif