/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MPEGRecorder.cpp,v 1.2 2004/02/14 12:01:09 gadibergman Exp $
 */

/************************************************************
			MPEGRecorder.cpp
			================

  Implementation of the MPEG Recorder MFC application class.
  The application installs a system-tray icon that sends
  message events to an invisible "dumb window for messages".

************************************************************/

#include "stdafx.h"
#include "MPEGRecorder.h"
#include "DumbWindowForMessages.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLameRecorderApp

BEGIN_MESSAGE_MAP(CLameRecorderApp, CWinApp)
	//{{AFX_MSG_MAP(CLameRecorderApp)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLameRecorderApp construction

CLameRecorderApp::CLameRecorderApp():
m_pDumbWnd(NULL)
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CLameRecorderApp object

CLameRecorderApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CLameRecorderApp initialization

BOOL CLameRecorderApp::InitInstance()
{
	if (!InitATL())
		return FALSE;

	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	if (cmdInfo.m_bRunEmbedded || cmdInfo.m_bRunAutomated)
	{
		return TRUE;
	}

	m_pDumbWnd = new CDumbWindowForMessages();

	if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

    // Initialize COM
	HRESULT hr = CoInitialize(NULL);

    if( FAILED( hr ) ) {
		AfxMessageBox(_T("Failed to initialize COM"), MB_OK, 0);
		return FALSE;
	}

	InstallIcon();

	m_pMainWnd = m_pDumbWnd;
	return TRUE;
}

int CLameRecorderApp::ExitInstance() 
{
	// TODO: Add your specialized code here and/or call the base class

	// Release COM
	CoUninitialize();
	
	if (m_bATLInited)
	{
		_Module.RevokeClassObjects();
		_Module.Term();
	
		CoUninitialize();
	}

	UninstallIcon();

	delete m_pDumbWnd;

	return CWinApp::ExitInstance();
}

#include <initguid.h>
#include "MPEGRecorder_i.c"
#include "MPEG4AVRecorder.h"
	
CMPEGRecorderModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
OBJECT_ENTRY(CLSID_MPEG4AVRecorder, CMPEG4AVRecorder)
END_OBJECT_MAP()

LONG CMPEGRecorderModule::Unlock()
{
	AfxOleUnlockApp();
	return 0;
}

LONG CMPEGRecorderModule::Lock()
{
	AfxOleLockApp();
	return 1;
}
LPCTSTR CMPEGRecorderModule::FindOneOf(LPCTSTR p1, LPCTSTR p2)
{
	while (*p1 != NULL)
	{
		LPCTSTR p = p2;
		while (*p != NULL)
		{
			if (*p1 == *p)
				return CharNext(p1);
			p = CharNext(p);
		}
		p1++;
	}
	return NULL;
}


BOOL CLameRecorderApp::InitATL()
{
	m_bATLInited = TRUE;

	HRESULT hRes = CoInitialize(NULL);

	if (FAILED(hRes))
	{
		m_bATLInited = FALSE;
		return FALSE;
	}

	_Module.Init(ObjectMap, AfxGetInstanceHandle());
	_Module.dwThreadID = GetCurrentThreadId();

	LPTSTR lpCmdLine = GetCommandLine(); //this line necessary for _ATL_MIN_CRT
	TCHAR szTokens[] = _T("-/");

	BOOL bRun = TRUE;
	LPCTSTR lpszToken = _Module.FindOneOf(lpCmdLine, szTokens);
	while (lpszToken != NULL)
	{
		if (lstrcmpi(lpszToken, _T("UnregServer"))==0)
		{
			_Module.UpdateRegistryFromResource(IDR_MPEGRECORDER, FALSE);
			_Module.UnregisterServer(TRUE); //TRUE means typelib is unreg'd
			bRun = FALSE;
			break;
		}
		if (lstrcmpi(lpszToken, _T("RegServer"))==0)
		{
			_Module.UpdateRegistryFromResource(IDR_MPEGRECORDER, TRUE);
			_Module.RegisterServer(TRUE);
			bRun = FALSE;
			break;
		}
		lpszToken = _Module.FindOneOf(lpszToken, szTokens);
	}

	if (!bRun)
	{
		m_bATLInited = FALSE;
		_Module.Term();
		CoUninitialize();
		return FALSE;
	}

	hRes = _Module.RegisterClassObjects(CLSCTX_LOCAL_SERVER, 
		REGCLS_MULTIPLEUSE);
	if (FAILED(hRes))
	{
		m_bATLInited = FALSE;
		CoUninitialize();
		return FALSE;
	}	

	return TRUE;

}

void CLameRecorderApp::InstallIcon()
{
	HICON hIcon = LoadIcon(IDR_MAINFRAME);

	char szTip[] = "MPEG Recorder";

	NOTIFYICONDATA nid = {
		sizeof(NOTIFYICONDATA),
		m_pDumbWnd->m_hWnd,
		0,
		NIF_ICON | NIF_MESSAGE | NIF_TIP,
		WM_USER,
		hIcon,
		NULL
	};

	strcpy(nid.szTip, szTip);

	Shell_NotifyIcon(NIM_ADD, &nid);
}

void CLameRecorderApp::UninstallIcon()
{
	NOTIFYICONDATA nid = {
		sizeof(NOTIFYICONDATA),
		m_pDumbWnd->m_hWnd,
		0,
		NIF_MESSAGE,
		0,
		0,
		NULL
	};

	Shell_NotifyIcon(NIM_DELETE, &nid);
}
