/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MediaDivision.h,v 1.1.1.1 2003/12/24 19:41:39 gadibergman Exp $
 */

/************************************************************
			MediaDivision.cpp
			=================

  A mechanism of dividing one media type to a number of
  media providers.

  Media frames/samples are written to each provider's
  standard output buffers object, according to the provider's
  registered play/pause times.

************************************************************/

#if !defined(AFX_MEDIADIVISION_H__B929458A_2E46_43DC_A715_9CB2C6CA15C6__INCLUDED_)
#define AFX_MEDIADIVISION_H__B929458A_2E46_43DC_A715_9CB2C6CA15C6__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// Includes
#include "mpeg4/combined/media_time.h"

// Classes declaration
class CMediaProvidersSet;
class TimeKeepingWriter;
class CTimeBasedMediaProvider;

class CMediaProvider
{
public:
	virtual ~CMediaProvider() {}
	virtual void PlayOrPause() = 0;
	virtual void Stop() = 0;
	virtual LONGLONG GetNumBytesProvided() const = 0;
};

class CMediaDivision  
{
private:
	CMediaProvidersSet* m_pvMediaProviders;

	unsigned long 
		m_ulAtomSize, 
		m_ulAtomsPerSec;

	LONGLONG m_ulBytesWritten;

	bool m_bHasStarted;
	Timestamp m_tsGlobalFirst, m_tsBufferFirst;
	bool m_bIsRelativeTimestamp;

public:
	CMediaDivision(
		unsigned long ulAtomSize, 
		unsigned long ulAtomsPerSec,
		bool bIsRelativeTimestamp
	);

	virtual ~CMediaDivision();

	CMediaProvider* CreateMediaProvider(TimeKeepingWriter* pWriter);

	unsigned long WriteBuffer(	const void * buf, 
								unsigned long len,
								Timestamp timestamp	);

	void DeclareWritingEnded();

	LONGLONG GetNumBytesWritten() const {
		return m_ulBytesWritten;
	}

	Timestamp GetLocalTimestamp() const {
		return GetTimestamp() - m_tsGlobalFirst;
	}

	Timestamp GetFirstBufferTimestamp() const {
		return m_tsBufferFirst;
	}
};

#endif // !defined(AFX_MEDIADIVISION_H__B929458A_2E46_43DC_A715_9CB2C6CA15C6__INCLUDED_)
