/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: Mp4Encoder.h,v 1.2 2004/03/27 13:47:16 gadibergman Exp $
 */

/************************************************************
			Mp4Encoder.h
			============

  An encapsulation of the MPEG4IP-mp4live encoder of
  audio samples and video frames into one mp4 file.

  The encoder makes use of:
  - an audio encoder that implements CAudioEncoder. Results
  have only been successfully observed using the 
  FAAC audio encoder.
  - a video encoder in two parts: RGB->YUV & YUV->XVID
  - a file mp4live based combination

************************************************************/

#if !defined(AFX_MP4ENCODER_H__0C0A0FDD_00FD_47F8_AC9F_3C9AF2F22595__INCLUDED_)
#define AFX_MP4ENCODER_H__0C0A0FDD_00FD_47F8_AC9F_3C9AF2F22595__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "MPEGEncoder.h"

class TimeKeepingWriter;
class CDXVideoReceiver;
class CDXSoundReceiver;
class CTimeBasedPipe;
class CAudioEncoder;
class CAudioEncoderGenerator;
class RGB2YUVConvertor;
class CXVidEncoder;
class CMp4Recorder;
class CFileNameGenerator;
class CMediaProvider;

class CMp4Encoder : public CMPEGEncoder
{
private:
	TimeKeepingWriter 
		*m_pAudioTrackWriter,
		*m_pVideoTrackWriter;

	CDXSoundReceiver * m_pSoundReceiver;
	CDXVideoReceiver * m_pVideoReceiver;

	CAudioEncoder *m_pAudioEncoder;
	RGB2YUVConvertor *m_pRGB2YUVConvertor;
	CXVidEncoder *m_pXVidEncoder;
	CMp4Recorder *m_pMp4Combination;

	CMediaProvider * m_pAudioProvider;
	CMediaProvider * m_pVideoProvider;

	CAudioEncoderGenerator & m_AudioEncoderGenerator;
	CFileNameGenerator & m_FileNameGenerator;

	double m_dDesiredFrameRate;

	CTimeBasedPipe
		*m_pPCMAudioPipe,
		*m_pRGBVideoPipe,
		*m_pYUVVideoPipe;

	void Init();


public:
	void GetProgress(
		double&	time,
		LONGLONG& pcmBytes, 
		LONGLONG& aacBytes, 
		LONGLONG& rgbBytes, 
		LONGLONG& yuvBytes, 
		LONGLONG& xvidBytes
		) const;

	CMp4Encoder(
		CDXSoundReceiver * pSoundReceiver,
		CDXVideoReceiver * pVideoReceiver,
		CAudioEncoderGenerator & audioEncoderGenerator,
		CFileNameGenerator & fileNameGenerator,
		double dDesiredFrameRate
		):
	m_pSoundReceiver(pSoundReceiver),
	m_pVideoReceiver(pVideoReceiver),
	m_AudioEncoderGenerator(audioEncoderGenerator),
	m_FileNameGenerator(fileNameGenerator),
	m_dDesiredFrameRate(dDesiredFrameRate),
	m_pPCMAudioPipe(NULL),
	m_pRGBVideoPipe(NULL),
	m_pYUVVideoPipe(NULL),
	m_pAudioTrackWriter(NULL),
	m_pVideoTrackWriter(NULL),
	m_pAudioEncoder(NULL),
	m_pRGB2YUVConvertor(NULL),
	m_pXVidEncoder(NULL),
	m_pMp4Combination(NULL),
	m_pAudioProvider(NULL),
	m_pVideoProvider(NULL)
	{
		Init();
	}

	virtual ~CMp4Encoder();

	void Stop();
	void PlayOrPause();
};

#endif // !defined(AFX_MP4ENCODER_H__0C0A0FDD_00FD_47F8_AC9F_3C9AF2F22595__INCLUDED_)
