/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RGBBlockCopier.cpp,v 1.2 2004/03/27 13:47:16 gadibergman Exp $
 */

/************************************************************
			RGBBlockCopier.cpp
			==================

  Reading and writing RGB frames, no actual processing done.

************************************************************/

#include "stdafx.h"
#include "TimeKeepingReaderWriter.h"
#include "RGBBlockCopier.h"
#include "Common/MutexProtection.h"
#include <process.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRGBBlockCopier::CRGBBlockCopier(
		TimeKeepingReader * pReader, 
		TimeKeepingWriter * pWriter, 
		unsigned long lFrameWidth, 
		unsigned long lFrameHeight
		):
	m_pReader(pReader),
	m_pWriter(pWriter),
	m_lFrameWidth(lFrameWidth), 
	m_lFrameHeight(lFrameHeight),
	m_hThreadTermination(CreateEvent(NULL, FALSE, TRUE, NULL))
{
	StartConversionThread();
}

CRGBBlockCopier::~CRGBBlockCopier()
{
	DWORD dwResult = WaitForSingleObject(m_hThreadTermination, INFINITE);
	ASSERT(dwResult == WAIT_OBJECT_0);

	CloseHandle(m_hThreadTermination);
}

void CRGBBlockCopier::PerformFiltering()
{
	unsigned long nFramePixels = m_lFrameWidth * m_lFrameHeight,
		nBufferSize = nFramePixels * 3;

	BYTE *pRGB = new BYTE[nBufferSize];

	while(true) 
	{
		Timestamp timestamp;
		Duration duration;

		unsigned int nRead = 
			m_pReader->ReadBytes(pRGB, nBufferSize, timestamp, duration); 

		if (!nRead)
			break;

		m_pWriter->WriteBytes(pRGB, nRead, timestamp, duration);
	}

	delete [] pRGB;

	m_pWriter->DeclareWritingEnded();
}

static void CRGBBlockCopier_ThreadFunc(LPVOID pParam) 
{
	CRGBBlockCopier * pCopier = (CRGBBlockCopier*) pParam;
	ThreadCompletionAnnouncement announce(pCopier->m_hThreadTermination);

	pCopier->PerformFiltering();
}

void CRGBBlockCopier::StartConversionThread() 
{
	_beginthread(CRGBBlockCopier_ThreadFunc, 0, this);
}
