/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RecorderDlg.h,v 1.2 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
			RecorderDlg.h
			=============

  Audio Recorder Dialog Box using MFC

  When the recorder pops up, it immediately starts encoding
  audio frames, using the audio encoder, generated by the
  audio encoder generator.

************************************************************/

#if !defined(AFX_RECORDERDLG_H__AD407D8F_C0B8_4B62_8A81_2F6DB11C0855__INCLUDED_)
#define AFX_RECORDERDLG_H__AD407D8F_C0B8_4B62_8A81_2F6DB11C0855__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// RecorderDlg.h : header file
//

// Classes declarations
class CTimeBasedPipe;
class CSoundReceiverBase;
class CFileWriter;
class CFileNameGenerator;
class CAudioEncoderGenerator;
class CAudioEncoder;
class CMediaProvider;

/////////////////////////////////////////////////////////////////////////////
// CRecorderDlg dialog

class CRecorderDlg : public CDialog
{
private:
	time_t m_tInitialTime;
	enum {
		CHANNELS = 2,
		BYTES_PER_SAMPLE = 2,
		SAMPLE_RATE = 44100
	};

	CFileNameGenerator & m_FileNameGenerator;
	CAudioEncoderGenerator & m_AudioEncoderGenerator;
	CFile * m_pFile;
	CFileWriter * m_pFileWriter;
	CAudioEncoder * m_pAudioEncoder;
	CSoundReceiverBase * m_pSoundReceiver;
	GUID m_gRecordingDeviceID;
	CMediaProvider * m_pSoundProvider;

	CBitmap 
		StopBitmap, 
		StopThenPlayBitmap, 
		StopThenRecordBitmap;

	enum FollowUp {
		NO_FOLLOWUP,
		PLAY_FILE,
		RECORD_NEW
	} m_FollowUp;

	void ReleaseStructures();

	CString m_sOriginalFileName;
	CString m_sDirectory;

	CRecorderDlg ** m_ppMe;
	bool& m_bInTransition;

// Construction
public:
	void Stop();
	~CRecorderDlg();
	CTimeBasedPipe* m_pPipe;
	CRecorderDlg(const GUID * pRecordingDeviceID,
		CFileNameGenerator & fileNameGenerator,
		CAudioEncoderGenerator & audioEncoderGenerator,
		bool& bInTransition,
		CRecorderDlg ** ppMe,
		CSoundReceiverBase * pSoundReceiver = NULL,
		CWnd* pParent = NULL);   // standard constructor

	enum {
		TIMER_EVENT_ID = 107
	};

// Dialog Data
	//{{AFX_DATA(CRecorderDlg)
	enum { IDD = IDD_RECORDING };
	CButton	m_StopButton;
	CButton	m_StopThenRecordButton;
	CButton	m_StopThenPlayButton;
	CString	m_sFileName;
	CString	m_sMP3Size;
	CString	m_sTime;
	CString	m_sWaveSize;
	CString	m_sEncodedSizeLabel;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CRecorderDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CRecorderDlg)
	afx_msg void OnStopButton();
	afx_msg void OnStopThenPlayButton();
	afx_msg void OnStopThenRecordButton();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
	afx_msg void OnClose();
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_RECORDERDLG_H__AD407D8F_C0B8_4B62_8A81_2F6DB11C0855__INCLUDED_)
