/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RecordingEventDialog.h,v 1.1.1.1 2003/12/24 19:41:38 gadibergman Exp $
 */

/************************************************************
			RecordingEventDialog.h
			======================

  A dialog box that enables the user to edit a repeatable
  recording event. This type of recording events is enabled
  if either the video viewer is closed or is held in control
  mode "externally planned".

************************************************************/

#if !defined(AFX_RECORDINGEVENTDIALOG_H__E0071B13_0ACF_4225_89F7_43E16B545A64__INCLUDED_)
#define AFX_RECORDINGEVENTDIALOG_H__E0071B13_0ACF_4225_89F7_43E16B545A64__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// RecordingEventDialog.h : header file
//

// Classes declaration
class CRepeatingTimeEvent;

/////////////////////////////////////////////////////////////////////////////
// CRecordingEventDialog dialog

class CRecordingEventDialog : public CDialog
{
// Construction
public:
	bool EditRecordingEvent(CRepeatingTimeEvent& tEvent);
	CRecordingEventDialog(CWnd* pParent = NULL);   // standard constructor

// Dialog Data
	//{{AFX_DATA(CRecordingEventDialog)
	enum { IDD = IDD_SCHEDULE_RECORDING };
	CListBox	m_DaysList;
	CString	m_csCurrentDate;
	CString	m_csCurrentTime;
	CTime	m_Date;
	int		m_iDayType;
	UINT	m_uiDurationInMinutes;
	CTime	m_EndTime;
	int		m_iFrameLimitType;
	CTime	m_StartTime;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CRecordingEventDialog)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CRecordingEventDialog)
	virtual BOOL OnInitDialog();
	virtual void OnOK();
	virtual void OnCancel();
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void FillDaysList();
	enum {
		DAY_IS_DATE,
		DAY_IS_WEEKDAY
	};

	enum {
		TIME_FRAME_LIMIT_IS_END_TIME,
		TIME_FRAME_LIMIT_IS_DURATION
	};

	CRepeatingTimeEvent* pTimeEvent;
	void SetTimeEvent(CRepeatingTimeEvent& tEvent) const;
	void SetByTimeEvent(const CRepeatingTimeEvent& tEvent);

	enum {
		MINUTE_TIMER_EVENT_ID = 100,
		MINUTE_IN_MS = 60 * 1000
	};

	unsigned int m_uiTimerID;

	void DisplayCurrentTime();
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_RECORDINGEVENTDIALOG_H__E0071B13_0ACF_4225_89F7_43E16B545A64__INCLUDED_)
