/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RecordingEventsUI.cpp,v 1.1.1.1 2003/12/24 19:41:42 gadibergman Exp $
 */

/************************************************************
			RecordingEventsUI.cpp
			=====================

  A dialog box that enables the user to add/edit/delete 
  repeatable recording events. It also enables viewing
  a chronological plan of the first occuring events.

  This type of recording events is enabled if either the 
  video viewer is closed or is held in control
  mode "externally planned".

************************************************************/

#include "stdafx.h"
#include "RecordingEventsUI.h"
#include "RepeatingTimeEvent.h"
#include "RecordingEventDialog.h"
#include "EventScheduleReport.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CRecordingEventsUI dialog


CRecordingEventsUI::CRecordingEventsUI(CWnd* pParent /*=NULL*/)
	: CDialog(CRecordingEventsUI::IDD, pParent)
{
	//{{AFX_DATA_INIT(CRecordingEventsUI)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	m_pvTimeEvents = NULL;
}


void CRecordingEventsUI::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRecordingEventsUI)
	DDX_Control(pDX, IDC_EVENTS_LIST, m_EventsList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRecordingEventsUI, CDialog)
	//{{AFX_MSG_MAP(CRecordingEventsUI)
	ON_BN_CLICKED(IDC_CLOSE, OnClose)
	ON_BN_CLICKED(IDC_DELETE_EVENT, OnDeleteEvent)
	ON_BN_CLICKED(IDC_EDIT_EVENT, OnEditEvent)
	ON_BN_CLICKED(IDC_NEW_EVENT, OnNewEvent)
	ON_BN_CLICKED(IDC_CHRONOLOGICAL_VIEW, OnChronologicalView)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRecordingEventsUI message handlers

void CRecordingEventsUI::OnClose() 
{
	EndDialog(0);
}

void CRecordingEventsUI::OnDeleteEvent() 
{
	int index = m_EventsList.GetCurSel();
	if (index != LB_ERR)
	{
		m_pvTimeEvents->erase(m_pvTimeEvents->begin() + index);
		UpdateListView();
	}
}

void CRecordingEventsUI::OnEditEvent() 
{
	int index = m_EventsList.GetCurSel();
	if (index != LB_ERR)
	{
		CRepeatingTimeEvent& rteEvent = (*m_pvTimeEvents)[index];
		CRecordingEventDialog dlgEditor;
		bool bRes = dlgEditor.EditRecordingEvent(rteEvent);
		if (bRes)
			UpdateListView();
	}
}

void CRecordingEventsUI::OnNewEvent() 
{
	CRepeatingTimeEvent rteEvent;
	CRecordingEventDialog dlgEditor;
	bool bRes = dlgEditor.EditRecordingEvent(rteEvent);
	if (bRes)
	{
		m_pvTimeEvents->push_back(rteEvent);
		UpdateListView();
	}
}

void CRecordingEventsUI::EditTimeEventsVector(CTimeEventsVector &vTimeEvents)
{
	m_pvTimeEvents = &vTimeEvents;
	DoModal();
	m_pvTimeEvents = NULL;
}

BOOL CRecordingEventsUI::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	UpdateListView();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CRecordingEventsUI::UpdateListView()
{
	m_EventsList.ResetContent();
	CTimeEventsVector::const_iterator iter;
	for (iter = m_pvTimeEvents->begin(); iter != m_pvTimeEvents->end(); ++iter)
	{
		const CRepeatingTimeEvent& tEvent = *iter;
		std::string strDescription;
		tEvent.GetString(strDescription);
		m_EventsList.AddString(strDescription.c_str());
	}
}

void CRecordingEventsUI::OnChronologicalView() 
{
	CEventScheduleReport reportDlg;
	reportDlg.ShowTimeSortedEvents(m_pvTimeEvents);
}
