/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: RenameFile.cpp,v 1.1.1.1 2003/12/24 19:41:41 gadibergman Exp $
 */

/************************************************************
			RenameFile.cpp
			==============

  A file renaming dialog box

************************************************************/

// PCH includes
#include "stdafx.h"

// Project includes
#include "RenameFile.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// RenameFile dialog


CRenameFile::CRenameFile(LPCTSTR sDirectory, LPCTSTR sOrigFileName,
						 LPCTSTR sNewFileName):
m_sDirectory(sDirectory)
{
	//{{AFX_DATA_INIT(CRenameFile)
	m_NewFile = _T("");
	m_OldFile = _T("");
	//}}AFX_DATA_INIT
	
	m_OldFile = sOrigFileName;
	m_NewFile = sNewFileName;

	Create(IDD, NULL);
}


void CRenameFile::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CRenameFile)
	DDX_Text(pDX, IDC_NEW_FILE_NAME, m_NewFile);
	DDX_Text(pDX, IDC_ORIGINAL_FILE_NAME, m_OldFile);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CRenameFile, CDialog)
	//{{AFX_MSG_MAP(CRenameFile)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CRenameFile message handlers

void CRenameFile::OnCancel() 
{
	// TODO: Add extra cleanup here
	delete this;	
	// CDialog::OnCancel();
}

void CRenameFile::OnOK() 
{
	// TODO: Add extra validation here
	if (Rename())
		delete this;

	// CDialog::OnOK();
}

bool CRenameFile::Rename()
{
	UpdateData(TRUE);

	CString sOriginalPath = m_sDirectory;
	sOriginalPath += m_OldFile;

	CString sNewPath = m_sDirectory;
	sNewPath += m_NewFile;
 
	if (sOriginalPath == sNewPath)
		return true;

	try {
		CFile::Rename(sOriginalPath, sNewPath);
		return true;
	}
	catch (CFileException ex) {
		TCHAR   szCause[255];
		CString strFormatted;

		ex.GetErrorMessage(szCause, 255);

		// (in real life, it's probably more
		// appropriate to read this from
		//  a string resource so it would be easy to
		// localize)

		strFormatted = _T("The data file could not be renamed because of this error: ");
		strFormatted += szCause;

		AfxMessageBox(strFormatted);

		return false;
	}
}
