/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SoftwarePropertiesPage.cpp,v 1.1.1.1 2003/12/24 19:41:39 gadibergman Exp $
 */

/************************************************************
			SoftwarePropertiesPage.cpp
			==========================

  A UI software properties page, a part of the properties
  sheet. It enables peeking an audio encoder, an AV file 
  format and a desired video frame rate.
  
************************************************************/

// PCH includes
#include "stdafx.h"

// Project includes
#include "MPEGRecorder.h"
#include "SoftwarePropertiesPage.h"
#include "SpecificApplicationData.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSoftwarePropertiesPage property page

IMPLEMENT_DYNCREATE(CSoftwarePropertiesPage, CPropertyPage)

CSoftwarePropertiesPage::CSoftwarePropertiesPage() : CPropertyPage(CSoftwarePropertiesPage::IDD)
{
	//{{AFX_DATA_INIT(CSoftwarePropertiesPage)
	m_AudioEncoderNumber = -1;
	m_VideoEncoderCombo = -1;
	//}}AFX_DATA_INIT

	m_VideoEncoderCombo = 0;
}

CSoftwarePropertiesPage::~CSoftwarePropertiesPage()
{
}

void CSoftwarePropertiesPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CSoftwarePropertiesPage)
	DDX_Control(pDX, IDC_AUDIO_BITRATE_COMBO, m_AudioKBitrateCombo);
	DDX_Control(pDX, IDC_VIDEO_FRAME_RATE_COMBO, m_VideoFrameRateCombo);
	DDX_Control(pDX, IDC_AVFormatCombo, m_AVFormatCombo);
	DDX_Radio(pDX, IDC_LAME_ENCODER, m_AudioEncoderNumber);
	DDX_Radio(pDX, IDC_MPEG4_DUMMY_RADIO, m_VideoEncoderCombo);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CSoftwarePropertiesPage, CPropertyPage)
	//{{AFX_MSG_MAP(CSoftwarePropertiesPage)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSoftwarePropertiesPage message handlers

void CSoftwarePropertiesPage::SetApplicationData(CSpecificApplicationData *pAppData)
{
	m_pAppData = pAppData;
	m_AudioEncoderNumber = pAppData->GetAudioEncoderNumber();
}

#define VALIDATE_COMBO_INDEX(i,combo,mess,str) \
	int i = combo.GetCurSel();\
	if (i < 0) {\
		AfxMessageBox(mess);\
		return;\
	}\
	CString str;\
	combo.GetLBText(i,str);

void CSoftwarePropertiesPage::OnOK() 
{
	UpdateData(TRUE);

	m_pAppData->SetAudioEncoderNumber(m_AudioEncoderNumber);

	VALIDATE_COMBO_INDEX(iAVFormatsIndex, m_AVFormatCombo, _T("No AV format selected"), sAVFormatName);

	m_pAppData->SetAVFormatName(sAVFormatName);

	VALIDATE_COMBO_INDEX(iAudioKBitRateIndex, m_AudioKBitrateCombo, _T("No Audio Bit Rate selected"), sAudioKBitRate);

	m_pAppData->SetAudioKBitRate(atoi(sAudioKBitRate));

	VALIDATE_COMBO_INDEX(iVideoFrameRateIndex, m_VideoFrameRateCombo, _T("No Video Frame Rate selected"), sVideoFrameRate);

	m_pAppData->SetVideoFrameRate(atoi(sVideoFrameRate));

	CPropertyPage::OnOK();
}

void CSoftwarePropertiesPage::FillAVFormatsCombo()
{
	m_AVFormatCombo.AddString("avi");
	m_AVFormatCombo.AddString("mp4");

	int iIndex = m_AVFormatCombo.FindString(-1, (LPCTSTR) m_pAppData->GetAVFormatName());
	if (iIndex == CB_ERR)
		iIndex = 0;

	m_AVFormatCombo.SetCurSel(iIndex);
}

BOOL CSoftwarePropertiesPage::OnInitDialog() 
{
	CPropertyPage::OnInitDialog();

	FillAVFormatsCombo();
	FillAudioBitRateCombo();
	FillVideoFrameRateCombo();
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CSoftwarePropertiesPage::FillAudioBitRateCombo()
{
	static DWORD vCommonKBitRates[] = {
		128, 160
	};

	DWORD dwAppBitRate = m_pAppData->GetAudioKBitRate();
	int iBitRateIndex(0);

	int iNumBitRates = sizeof(vCommonKBitRates) / sizeof(DWORD);
	for (int i = 0; i < iNumBitRates; ++i) {
		TCHAR vNumBuf[100];
		_stprintf(vNumBuf, _T("%d"), vCommonKBitRates[i]);
		m_AudioKBitrateCombo.AddString(vNumBuf);
		if (vCommonKBitRates[i] == dwAppBitRate)
			iBitRateIndex = i;
	}

	m_AudioKBitrateCombo.SetCurSel(iBitRateIndex);
}

void CSoftwarePropertiesPage::FillVideoFrameRateCombo()
{
	DWORD dwAppFrameRate = m_pAppData->GetVideoFrameRate();
	int iFrameRateIndex(0);

	for (int i = 0, r = 5; r <= 25; ++i, r += 5) {
		TCHAR vNumBuf[100];
		_stprintf(vNumBuf, _T("%d"), r);
		m_VideoFrameRateCombo.AddString(vNumBuf);
		if (r == dwAppFrameRate)
			iFrameRateIndex = i;
	}

	m_VideoFrameRateCombo.SetCurSel(iFrameRateIndex);
}
