/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SoundCaptureDevices.cpp,v 1.1.1.1 2003/12/24 19:41:38 gadibergman Exp $
 */

/************************************************************
			SoundCaptureDevices.cpp
			=======================

  Enumeration of DirectX sound capture devices
  
************************************************************/


#include "stdafx.h"
#include "MPEGRecorder.h"
#include "SoundCaptureDevices.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Name: DSoundEnumCallback()
// Desc: Enumeration callback called by DirectSoundEnumerate
//-----------------------------------------------------------------------------
static INT_PTR CALLBACK DSoundEnumCallback( GUID* pGUID, LPSTR strDesc, LPSTR strDrvName,
                                  VOID* pContext )
{
    CSoundCaptureDevices * pDevices = (CSoundCaptureDevices *) pContext;
	pDevices->AddRecordingDeviceID(pGUID, strDesc);

	return TRUE;
}

void CSoundCaptureDevices::Enumerate()
{
    // Enumerate the capture devices and place them in the combo box
    DirectSoundCaptureEnumerate( (LPDSENUMCALLBACK)DSoundEnumCallback,
                                 (VOID*) this );

}

void CSoundCaptureDevices::AddRecordingDeviceID(GUID *pRecordingDeviceID, LPSTR strDesc)
{
	if (!pRecordingDeviceID)
		return;

	RecordingDevice r;
	r.m_ID = *pRecordingDeviceID;
	r.m_sDesc = strDesc;

	m_vDevices.push_back(r);
}

bool CSoundCaptureDevices::GetDeviceIDByName(LPCTSTR devName, GUID & devID) {
	DevicesVector::const_iterator iter;
	for (iter = m_vDevices.begin(); iter != m_vDevices.end(); ++iter) {
		const RecordingDevice & dev = (*iter);
		if (dev.m_sDesc == devName) {
			devID = dev.m_ID;
			return true;
		}
	}
	return false;
}

