/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: SpecificApplicationData.h,v 1.2 2004/01/25 17:45:33 gadibergman Exp $
 */

/************************************************************
			SpecificApplicationData.h
			=========================

  The persistency of the application properties.
  This object contains all application variables that 
  need to be saved between sessions, loads them from the
  registry when the application starts to run and saves them
  to the registry every time the user presses "ok" in the 
  properties sheet UI.
  
************************************************************/

#ifndef __SPECIFIC_APPLICATION_DATA
#define __SPECIFIC_APPLICATION_DATA

class CSpecificApplicationData {

private:
	bool UniteWithRegistryValues(bool bSave);
	static bool UniteWithRegistryKey(HKEY hRegKey, bool bSave, LPCTSTR valuename, CString & value);
	static bool UniteWithRegistryKey(HKEY hRegKey, bool bSave, LPCTSTR valuename, DWORD & value);
	static bool GetRegistryValue(HKEY hRegKey, LPCTSTR valuename, CString & value);
	static bool GetRegistryValue(HKEY hRegKey, LPCTSTR valuename, DWORD & value);
	static bool SetRegistryValue(HKEY hRegKey, LPCTSTR valuename, const CString & value);
	static bool SetRegistryValue(HKEY hRegKey, LPCTSTR valuename, DWORD value);

	CString 
		m_sRecordingDirectory,
		m_sRecordingDeviceName,
		m_sVideoCaptureDevName,
		m_sAVFormatName,
		m_sPreferredImageSize;

	DWORD
		m_uAudioEncoderNumber,
		m_uTVVideoFormatNumber,
		m_uTVChannelNumber,
		m_uAudioKBitRate,
		m_uVideoFrameRate;

public:
	CSpecificApplicationData():
	m_sRecordingDirectory(_T("C:\\")),
	m_sAVFormatName(_T("avi")),
	m_sPreferredImageSize(_T("default")),
	m_uAudioEncoderNumber(0),
	m_uTVVideoFormatNumber(0),
	m_uTVChannelNumber(0),
	m_uVideoFrameRate(10),
	m_uAudioKBitRate(160)
	{}

	const CString & GetRecordingDirectory() const {
		return m_sRecordingDirectory;
	}
	
	const CString & GetRecordingDeviceName() const {
		return m_sRecordingDeviceName;
	}

	const CString & GetVideoCaptureDevName() const {
		return m_sVideoCaptureDevName;
	}

	const CString & GetAVFormatName() const {
		return m_sAVFormatName;
	}

	const CString & GetPreferredImageSize() const {
		return m_sPreferredImageSize;
	}

	DWORD GetAudioEncoderNumber() const {
		return m_uAudioEncoderNumber;
	}

	DWORD GetTVVideoFormatNumber() const {
		return m_uTVVideoFormatNumber;
	}

	DWORD & GetTVChannelRef() {
		return m_uTVChannelNumber;
	}

	DWORD GetAudioKBitRate() const {
		return m_uAudioKBitRate;
	}

	DWORD GetVideoFrameRate() const {
		return m_uVideoFrameRate;
	}

	void SetRecordingDirectory(const CString & sRecordingDirectory) {
		m_sRecordingDirectory = sRecordingDirectory;
	}

	void SetRecordingDeviceName(const CString & sRecordingDeviceName) {
		m_sRecordingDeviceName = sRecordingDeviceName;
	}

	void SetVideoCaptureDevName(const CString & sVideoCaptureDevName) {
		m_sVideoCaptureDevName = sVideoCaptureDevName;
	}

	void SetAVFormatName(const CString & sAVFormatName) {
		m_sAVFormatName = sAVFormatName;
	}

	void SetPreferredImageSize(const CString & sPreferredImageSize) {
		m_sPreferredImageSize = sPreferredImageSize;
	}

	void SetAudioEncoderNumber(DWORD uAudioEncoderNumber) {
		m_uAudioEncoderNumber = uAudioEncoderNumber;
	}

	void SetTVVideoFormatNumber(DWORD uTVVideoFormatNumber) {
		m_uTVVideoFormatNumber = uTVVideoFormatNumber;
	}

	void SetAudioKBitRate(DWORD uAudioKBitRate) {
		m_uAudioKBitRate = uAudioKBitRate;
	}

	void SetVideoFrameRate(DWORD uVideoFrameRate) {
		m_uVideoFrameRate = uVideoFrameRate;
	}

	bool Load();
	void Save();
};

#define DEFAULT_PREF_IMAGE_SIZE "default"
#define BEST_PREF_IMAGE_SIZE	"best"

#endif
