/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: StreamReceiver.cpp,v 1.1.1.1 2003/12/24 19:41:40 gadibergman Exp $
 */

/************************************************************
			StreamReceiver.cpp
			==================

  The base classes of the audio and video input objects.
  
************************************************************/

// PCH includes
#include "StdAfx.h"

// Common Includes
#include "Common/MutexProtection.h"

// C includes
#include <assert.h>

// Project includes
#include "StreamReceiver.h"
#include "TimeKeepingReaderWriter.h"
#include "MediaDivision.h"

CStreamReceiver::CStreamReceiver():
m_pMediaDivision(NULL)
{}

CStreamReceiver:: ~CStreamReceiver() {
	delete m_pMediaDivision;
}

unsigned long CStreamReceiver::
WriteBuffer(	const void * buf, 
				unsigned long len,
				Timestamp timestamp	)
{
	unsigned long ulBytesWritten = 
		m_pMediaDivision->WriteBuffer(buf, len, timestamp);
	
	return ulBytesWritten;
}

void CStreamReceiver::CreateMediaDivision
	(	
		unsigned long ulAtomSize,
		unsigned long ulAtomsPerSec,
		bool bIsRelativeTimestamp
	)
{
	assert(!m_pMediaDivision);

	m_pMediaDivision = new CMediaDivision(
		ulAtomSize, 
		ulAtomsPerSec,
		bIsRelativeTimestamp
	);
}

void CStreamReceiver::DeclareWritingEnded()
{
	if (m_pMediaDivision)
		m_pMediaDivision->DeclareWritingEnded();
}

CMediaProvider* CStreamReceiver::
CreateMediaProvider(TimeKeepingWriter* pWriter)
{
	assert(m_pMediaDivision);
	return m_pMediaDivision->CreateMediaProvider(pWriter);
}

