/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: StreamReceiver.h,v 1.1.1.1 2003/12/24 19:41:39 gadibergman Exp $
 */

/************************************************************
			StreamReceiver.h
			================

  The base classes of the audio and video input objects.
  
************************************************************/

#ifndef __STREAM_RECEIVER_H
#define __STREAM_RECEIVER_H

// Classes declaration
class CMediaDivision;
class CMediaProvider;
class TimeKeepingWriter;

#include "mpeg4/combined/media_time.h"

class CStreamReceiver {
protected:
	unsigned long WriteBuffer(	const void * buf, 
								unsigned long len,
								Timestamp timestamp	);

	void CreateMediaDivision(	unsigned long ulAtomSize,
								unsigned long ulAtomsPerSec,
								bool bIsRelativeTimestamp
							);

	void DeclareWritingEnded();

public:
	CStreamReceiver();

	virtual ~CStreamReceiver();

	virtual void Stop() = 0;

	CMediaProvider* CreateMediaProvider(TimeKeepingWriter* pWriter);

private:

	CMediaDivision* m_pMediaDivision;

};

class CSoundReceiverBase : public CStreamReceiver {
public:
	CSoundReceiverBase() {}

	virtual void Start() = 0;

	virtual ~CSoundReceiverBase() {}
};

class CVideoReceiverBase : public CStreamReceiver {
public:
	CVideoReceiverBase() {}

	virtual ~CVideoReceiverBase() {}
};

#endif
