/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TimeBasedPipe.cpp,v 1.1.1.1 2003/12/24 19:41:40 gadibergman Exp $
 */

/************************************************************
			TimeBasedPipe.cpp
			=================

  A class that derives the blocks pipe allowing every
  block to store its time-stamp and duration.
  These terms were borrowed from the MPEG4IP project.
  
************************************************************/


#include "stdafx.h"
#include "MPEGRecorder.h"
#include "TimeBasedPipe.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTimeBasedPipe::CTimeBasedPipe()
{

}

CTimeBasedPipe::~CTimeBasedPipe()
{

}

CTimeBasedPipeBlock::CTimeBasedPipeBlock(
	const void * pBuffer, 
	unsigned long uSize,
	Timestamp timestamp, 
	Duration duration
	):
CBasicPipeBlock(pBuffer, uSize),
m_uSize(uSize), 
m_uPos(0),
m_timestamp(timestamp),
m_duration(duration)
{
}

unsigned int CTimeBasedPipeBlock::Read(
	void * buf, 
	unsigned int numBytes,
	Timestamp & timestamp,
	Duration & duration
	)
{
	unsigned int bytesRead = CBasicPipeBlock::Read(buf, numBytes);
	timestamp = m_timestamp + (u_int64_t) m_uPos * m_duration / m_uSize;
	duration = (u_int64_t) bytesRead * m_duration / m_uSize;
	m_uPos += bytesRead;

	return bytesRead;
}

unsigned int CTimeBasedPipe::Read(
	void * buf, 
	unsigned int numBytes,
	Timestamp & timestamp,
	Duration & duration
	)
{
	m_ReadingDuration = 0;
	m_ReadingTimestamp = 0;

	unsigned int bytesRead = 
		CBlocksPipe::Read(buf, numBytes);

	timestamp = m_ReadingTimestamp;
	duration = m_ReadingDuration;

	return bytesRead;
}

unsigned int CTimeBasedPipe::ReadBlock(
	CBasicPipeBlock * pBlock, 
	void * buf, 
	unsigned int numBytes
	)
{
	CTimeBasedPipeBlock * pTimedBlock = (CTimeBasedPipeBlock*)
		pBlock;

	Timestamp timestamp;
	Duration duration;

	unsigned int bytesRead = 
		pTimedBlock->Read(buf, numBytes, timestamp, duration);

	if (!m_ReadingTimestamp)
		m_ReadingTimestamp = timestamp;

	m_ReadingDuration = duration + timestamp - m_ReadingTimestamp;

	return bytesRead;
}

unsigned int CTimeBasedPipe::Write(
	const void * buf, 
	unsigned int numBytes,
	Timestamp timestamp,
	Duration duration
	)
{
	m_WritingTimestamp = timestamp;
	m_WritingDuration = duration;
	m_WritingSize = numBytes;
	m_WritingPos = 0;

	return CBlocksPipe::Write(buf, numBytes);
}

CBasicPipeBlock * CTimeBasedPipe::CreateBasicBlock(
	const void * pBuffer, 
	unsigned long uSize
	)
{
	Timestamp timestamp = m_WritingTimestamp;
	if (m_WritingPos)
		timestamp += (u_int64_t) m_WritingPos * m_WritingDuration / m_WritingSize;

	Duration duration = (m_WritingSize == uSize) ? m_WritingDuration :
		m_WritingDuration * uSize / m_WritingSize;

	CTimeBasedPipeBlock * pBlock = 
		new CTimeBasedPipeBlock(pBuffer, uSize, 
		timestamp, duration);

	m_WritingPos += uSize;

	return pBlock;
}

