/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TimeBasedPipe.h,v 1.2 2004/01/25 17:45:33 gadibergman Exp $
 */

/************************************************************
			TimeBasedPipe.h
			===============

  A class that derives the blocks pipe allowing every
  block to store its time-stamp and duration.
  These terms were borrowed from the MPEG4IP project.
  
************************************************************/

#if !defined(AFX_TIMEBASEDPIPE_H__5E193030_A1E2_4667_AA71_70A00590A3A4__INCLUDED_)
#define AFX_TIMEBASEDPIPE_H__5E193030_A1E2_4667_AA71_70A00590A3A4__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "BlocksPipe.h"
#include "TimeKeepingReaderWriter.h"

class CTimeBasedPipeBlock : public CBasicPipeBlock {
private:
	Timestamp m_timestamp;
	Duration m_duration;
	unsigned int m_uSize, m_uPos;

public:
	CTimeBasedPipeBlock(const void * pBuffer, unsigned long uSize,
		Timestamp timestamp, Duration duration);

	unsigned int Read(
		void * buf, 
		unsigned int numBytes,
		Timestamp & timestamp,
		Duration & duration
		);

	virtual ~CTimeBasedPipeBlock() {}
};

class CTimeBasedPipe : 
	private CBlocksPipe, 
	public TimeKeepingReader, 
	public TimeKeepingWriter  
{
private:
	unsigned int Read(
		void * buf, 
		unsigned int numBytes,
		Timestamp & timestamp,
		Duration & duration
		);

	unsigned int Write(
		const void * buf, 
		unsigned int numBytes,
		Timestamp timestamp,
		Duration duration
		);

	CBasicPipeBlock * CreateBasicBlock(const void * pBuffer, unsigned long uSize);
	unsigned int ReadBlock(CBasicPipeBlock * pBlock, void * buf, unsigned int numBytes);

	void DeclareWritingEnded() {
		CBlocksPipe::DeclareWritingEnded();
	}

	Timestamp m_ReadingTimestamp, m_WritingTimestamp;
	Duration m_ReadingDuration, m_WritingDuration;
	unsigned int m_WritingPos, m_WritingSize;

public:
	CTimeBasedPipe();
	virtual ~CTimeBasedPipe();

	LONGLONG GetNumBytesWritten() { 
		return TimeKeepingWriter::GetNumBytesWritten();
	}

	void SetAllocationSizeLimit(unsigned long uAllocationSizeLimit)
	{
		CBlocksPipe::SetAllocationSizeLimit(uAllocationSizeLimit);
	}

};

#endif // !defined(AFX_TIMEBASEDPIPE_H__5E193030_A1E2_4667_AA71_70A00590A3A4__INCLUDED_)
