/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: TimeDependentFileNameGenerator.cpp,v 1.1.1.1 2003/12/24 19:41:40 gadibergman Exp $
 */

/************************************************************
			TimeDependentFileNameGenerator.cpp
			==================================

  A file name and directory generator, that uses a root 
  directory, and generates sub-directories based on the date
  and files based on the time.
  
************************************************************/

// PCH includes
#include "StdAfx.h"

// MS C includes
#include <direct.h>

// Project includes
#include "TimeDependentFileNameGenerator.h"


void CTimeDependentFileNameGenerator::
GenerateFileName(CString & sDirectory, CString & sFileName) {

	sDirectory = m_sRootDirectory;

	// Get current time
	CTime t = CTime::GetCurrentTime();

	unsigned int
		day = t.GetDay(),
		month = t.GetMonth(),
		year = t.GetYear(),
		hour = t.GetHour(),
		minute = t.GetMinute(),
		second = t.GetSecond();

	TCHAR sDayDirectory[50];
	_stprintf(sDayDirectory, "%04d_%02d_%02d", year, month, day);
	
	sDirectory += sDayDirectory;

	// Try to create this directory
	if ( (_tmkdir((LPCTSTR) sDirectory) == -1) && (errno == ENOENT) )
	{
		AfxMessageBox("cannot create recording directory!");
		return;
	}

	sDirectory += _T("\\");

	TCHAR sTempFileName[50];
	_stprintf(sTempFileName, "%02d_%02d_%02d%s", 
		hour, minute, second, (LPCTSTR) m_sExtension);

	sFileName = sTempFileName;
}

