/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: VideoCaptureDevices.cpp,v 1.1.1.1 2003/12/24 19:41:37 gadibergman Exp $
 */

/************************************************************
			VideoCaptureDevices.cpp
			=======================

  Enumeration of DirectX video input devices.
  
************************************************************/

// PCH includes
#include "stdafx.h"

// Project includes
#include "MPEGRecorder.h"
#include "VideoCaptureDevices.h"

// STL includes
#include <string>

using namespace std;

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

#define TCE(f,errmess) \
{\
	HRESULT hr = (f);\
	if (hr != NOERROR)\
		throw new string(errmess);\
}

CVideoCaptureDevices::CVideoCaptureDevices()
{
	HRESULT hr;

    // enumerate all video capture devices
    ICreateDevEnum *pCreateDevEnum;
    TCE(CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
		IID_ICreateDevEnum, (void**)&pCreateDevEnum),
		"Error Creating Device Enumerator");

    IEnumMoniker *pEm;
    TCE(pCreateDevEnum->CreateClassEnumerator(CLSID_VideoInputDeviceCategory,
		&pEm, 0),
		"No video capture hardware detected");

    pEm->Reset();
    ULONG cFetched;
    IMoniker *pM;
    while(hr = pEm->Next(1, &pM, &cFetched), hr==S_OK)
    {
	IPropertyBag *pBag;
	hr = pM->BindToStorage(0, 0, IID_IPropertyBag, (void **)&pBag);
	if(SUCCEEDED(hr)) {
	    VARIANT var;
	    var.vt = VT_BSTR;
	    hr = pBag->Read(L"FriendlyName", &var, NULL);
	    if (hr == NOERROR) {
			IBaseFilter * devFilter;
			// Get device name and capture filter
			hr = pM->BindToObject( 0, 0, IID_IBaseFilter, (void**) &devFilter);
			if (devFilter) {
			    USES_CONVERSION;
				m_DevicesList.push_back(CVidCapDevDesc(W2T(var.bstrVal),
					devFilter));
			}

			SysFreeString(var.bstrVal);
            pM->AddRef();
	    }
	    pBag->Release();
	}
	pM->Release();
    }
    pEm->Release();
}

CVideoCaptureDevices::~CVideoCaptureDevices()
{
}

IBaseFilter * CVideoCaptureDevices::GetVideoFilterByName(LPCTSTR sName) const
{
	CDevicesList::const_iterator iter;

	for (iter = m_DevicesList.begin(); iter != m_DevicesList.end(); ++iter) {
		const CVidCapDevDesc & dev = *iter;
		const CString & devName = dev.GetDeviceName();
		if (devName == sName)
			return dev.GetDeviceFilter();
	}

	return NULL;
}
