/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: VideoCodecTrackWriter.h,v 1.1.1.1 2003/12/24 19:41:39 gadibergman Exp $
 */

/************************************************************
			VideoCodecTrackWriter.h
			=======================

  The track writer of the MPEG4 video codec, used with
  the FFMPEG AVFormat encoder.
    
************************************************************/

#ifndef __VIDEO_CODEC_TRACK_WRITER_H
#define __VIDEO_CODEC_TRACK_WRITER_H

// Includes
#include "TimeKeepingReaderWriter.h"
#include "mpeg4/combined/media_time.h"
#include "ffmpeg/avformat.h"

// Classes declarations
class FFMPEGAccess;

class CVideoCodecTrackWriter : public TimeKeepingWriter {
private:
	CAVFormatWriter &m_AVFormatWriter;
    AVCodec *m_pAVCodec;
	AVFrame *m_pFrame, *m_pOriginalFrame;
	uint8_t *m_pOutBuffer, *m_pConversionBuffer;
	AVStream *m_pStream;	
	int m_iOutBufferSize;
	AVCodecContext *m_pCodecContext;
	FFMPEGAccess &m_ffmpeg;
	int m_iPictureSize, m_iOriginalSize;
	PixelFormat m_pfIn;
	unsigned long m_uNumFrames;
	Timestamp m_timestamp;
	void *m_pInitialTransformBuffer;

	LONGLONG 
		m_uYUVBytesWritten,
		m_uXVidBytesWritten;

	enum {
		INITIAL_BYTES_PER_PIXEL	= 3
	};

	unsigned int Write(
		const void * buf, 
		unsigned int numBytes,
		Timestamp timestamp,
		Duration duration
		) ;

public:
	CVideoCodecTrackWriter(
		CAVFormatWriter& avFormatWriter,
		AVFormatContext *pFormatContext,
		AVStream* pAVStream,
		PixelFormat pfIn);

	virtual ~CVideoCodecTrackWriter();
	void DeclareWritingEnded();

	LONGLONG GetNumYUVBytesWritten() const {
		return m_uYUVBytesWritten;
	}

	LONGLONG GetNumXVidBytesWritten() const {
		return m_uXVidBytesWritten;
	}

};


#endif //__VIDEO_CODEC_TRACK_WRITER_H
