/*
 * MPEG Recorder
 * Copyright (c) 2003 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: VideoViewer.h,v 1.6 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
			VideoViewer.h
			=============

  - UI for recording combined video+audio file
  - UI for TV and cross-bar control
  - UI for basic scheduling of a recording start/stop time
  
************************************************************/

#if !defined(AFX_VIDEOVIEWER_H__A2080B35_D3F4_41A2_97CA_CECD3C242AC1__INCLUDED_)
#define AFX_VIDEOVIEWER_H__A2080B35_D3F4_41A2_97CA_CECD3C242AC1__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// VideoViewer.h : header file
//

class CDummySinkFilter;
struct IBaseFilter;
class CDXVideoReceiver;
class CDXSoundReceiver;
class CFileNameGenerator;
class CMPEGEncoder;
class CAudioEncoderGenerator;
class CExternalVideoViewer;
struct FilterGraphWrapper;

#include "Common/MutexProtection.h"

/////////////////////////////////////////////////////////////////////////////
// CVideoViewer dialog

class CVideoViewer : public CDialog
{
// Construction
public:
	enum RecorderControlMode {
		MANUAL_CONTROL,
		EXTERNAL_CONTROL,
		DURATION_CONTROL,
		TIME_CONTROL
	};

	// Destructor
	virtual  ~CVideoViewer();

	// Main and only constructor
	CVideoViewer(
		
		// Video device DirectX filter
		IBaseFilter * pVideoIn,

		// Audio device id
		const GUID & audioRecordingDeviceID,

		// Audio encoder generator
		CAudioEncoderGenerator & audioEncoderGenerator,

		// File name generator
		CFileNameGenerator & fileNameGenerator,

		// Control Mode
		RecorderControlMode controlMode,

		// Analog video TV format
		long lAnalogVideoStandard,

		// TV channel number (reference)
		DWORD & lChannelNumberRef,

		// Desired frame rate (#frames per second)
		unsigned int uDesiredFrameRate,

		// Desired frame size
		const std::string & strPreferredImageSize,

		// Is the dialog box in transition from construction to init
		bool& bInTransition,

		// Address of pointer to this structure
		// since this object deletes itself
		CVideoViewer ** ppMe,

		// Parent window
		CWnd* pParent
		
		);

	CDXVideoReceiver* GetVideoReceiver() const 
	{
		return m_pVideoReceiver;
	}

	void RestoreInternalPreview();
	void ExternalStopRecording();
	void ExternalStartRecording();
	void ManualStartRecording();
	void OnMinutePassed();

// Dialog Data
	//{{AFX_DATA(CVideoViewer)
	enum { IDD = IDD_VIDEO_VIEWER };
	CButton	m_VideoButton;
	CButton	m_TVAudioButton;
	CButton	m_PauseButton;
	CButton	m_ExternalVideoWindowButton;
	CButton	m_CrossbarButton;
	CButton	m_StopThenRecordButton;
	CButton	m_StopButton;
	CComboBox	m_TunerModesCombo;
	CButton	m_TVButton;
	CSliderCtrl	m_ChannelsSlider;
	long	m_lChannelNumber;
	CString	m_sAACBytes;
	CString	m_sPCMBytes;
	CString	m_sRGBBytes;
	CString	m_sXVidBytes;
	CString	m_sYUVBytes;
	CString	m_sSampleRate;
	CString	m_sEncodedBytesLabel;
	CString	m_sTime;
	CString	m_sFrameInformation;
	CTime	m_DialogEventTime;
	UINT	m_uiDialogEventRemainingMinutes;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CVideoViewer)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL OnCommand(WPARAM wParam, LPARAM lParam);
	virtual void PostNcDestroy();
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CVideoViewer)
	afx_msg void OnCloseButton();
	virtual BOOL OnInitDialog();
	afx_msg void OnDestroy();
	afx_msg void OnClose();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnTVProperties();
	afx_msg void OnChangeChannelNumber();
	afx_msg void OnSelchangeTunerModesCombo();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnStop();
	afx_msg void OnRecord();
	afx_msg void OnCrossbarProperties();
	afx_msg void OnPressedExternalWindowButton();
	afx_msg void OnPause();
	afx_msg void OnManualRecSelected();
	afx_msg void OnEventTimeSelected();
	afx_msg void OnExternalPlanSelected();
	afx_msg void OnEventMinutesSelected();
	afx_msg void OnCaptureImage();
	afx_msg void OnTVAudioProperties();
	afx_msg void OnVideoProperties();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void StopRecording();
	void StartRecording();
	void PerformDefaultRecordingEvent();
	void SetControlMode(RecorderControlMode mode);
	CMPEGEncoder* CreateMPEGEncoder();
	void FindCrossbar();
	void LookForAudio();
	static void SetAnalogVideoFormat(IBaseFilter *pVideoFilter, long lAnalogVideoStandard);
	void EnumerateTunerModes();
	void SetTVChannel();
	static void ShowProperties(HWND hWnd, IUnknown * pObject, ISpecifyPropertyPages *pSpec);
	static ISpecifyPropertyPages * GetPropertiesPages(IUnknown * pObject);
	ISpecifyPropertyPages 
		* m_pTVProps, 
		* m_pCrossbarProps,
		* m_pTVAudioProps,
		* m_pVideoProps;
	void InitTVControl();
	IAMTVTuner* m_pTVTuner;
	IAMCrossbar* m_pCrossbar;
	IAMTVAudio* m_pTVAudio;
	void GetTVTuner();
	void RenderUnconnectedCrossbarOutputPins();
	void Stop();
	void EnableSpecialDeviceProperties();
	CDXVideoReceiver* m_pVideoReceiver;
	CDXSoundReceiver* m_pSoundReceiver;
	IBaseFilter* m_pVideoFilter;
	GUID m_AudioRecordingDeviceID;
	CFileNameGenerator & m_FileNameGenerator;
	CAudioEncoderGenerator & m_AudioEncoderGenerator;
	long m_lAnalogVideoStandard;
	DWORD & m_lChannelNumberRef;
	CMPEGEncoder * m_pMPEGEncoder;
	FilterGraphWrapper* m_pFGW;
	CDummySinkFilter* m_pDummyFilter;
	bool m_bInitSuccess;

	std::string m_strPreferredImageSize;

	unsigned int m_uDesiredFrameRate;

	enum {
		CHANNELS = 2,
		BYTES_PER_SAMPLE = 2,
		SAMPLE_RATE = 44100
	};

	enum {
		TIMER_EVENT_ID = 107
	};

	CBitmap
		m_PauseBitmap,
		m_StopBitmap,
		m_StopThenRecordBitmap;

	CExternalVideoViewer* m_pExternalVideoViewer;

	Mutex m_EncoderPtrMutex;

	bool& m_bInTransition;
	CVideoViewer ** m_ppMe;

	RecorderControlMode m_ControlMode;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_VIDEOVIEWER_H__A2080B35_D3F4_41A2_97CA_CECD3C242AC1__INCLUDED_)
