/*
 * MPEG Recorder
 * Copyright (c) 2004 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: VolumeMeterDialog.cpp,v 1.1 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
			VolumeMeterDialog.cpp
			=====================

  A dialog box for displaying a volume meter.
  The input sound signal is assumed 16 bits stereo.
  
************************************************************/

// PCH includes
#include "stdafx.h"

// STL includes
#include <string>

// Project includes
#include "DXSoundReceiver.h"
#include "TimeBasedPipe.h"
#include "VolumeMeterDialog.h"
#include "VolumeProbe.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVolumeMeterDialog dialog


CVolumeMeterDialog::CVolumeMeterDialog(
	const GUID * pGdRecordingDeviceID,
	bool& bInTransition,
	CVolumeMeterDialog** ppMe,
	CWnd* pParent
	):
CDialog(CVolumeMeterDialog::IDD, pParent),
m_bInTransition(bInTransition),
m_ppMe(ppMe),
m_pPipe(NULL),
m_pVolumeProbe(NULL),
m_pSoundReceiver(NULL)
{
	//{{AFX_DATA_INIT(CVolumeMeterDialog)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	try
	{
		m_pSoundReceiver = new CDXSoundReceiver(
			pGdRecordingDeviceID,
			CHANNELS, 
			BYTES_PER_SAMPLE, 
			SAMPLE_RATE
			);

		m_pPipe = new CTimeBasedPipe;

		m_pVolumeProbe = new CVolumeProbe;

		m_bInTransition = true;

		Create(IDD, pParent);
	}
	catch (std::string* pstrError)
	{
		AfxMessageBox(pstrError->c_str());
		FreeInternalObjects();
		throw pstrError;
	}
}

CVolumeMeterDialog::~CVolumeMeterDialog()
{
	FreeInternalObjects();

	*m_ppMe = NULL;
}

void CVolumeMeterDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CVolumeMeterDialog)
	DDX_Control(pDX, IDC_PROGRESS_2, m_pcProgress2);
	DDX_Control(pDX, IDC_PROGRESS_1, m_pcProgress1);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CVolumeMeterDialog, CDialog)
	//{{AFX_MSG_MAP(CVolumeMeterDialog)
	ON_BN_CLICKED(IDC_CLOSE, OnCloseButton)
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVolumeMeterDialog message handlers

LRESULT CVolumeMeterDialog::WindowProc(UINT message, WPARAM wParam, LPARAM lParam) 
{
	if ( message == CVolumeProbe::VOLUME_UNIT_READY )
	{
		m_pcProgress1.SetPos( wParam );
		m_pcProgress2.SetPos( lParam );
		
		return 0;
	}

	if ( message == CVolumeProbe::END_OF_STREAM )
	{
		DestroyWindow();
		delete this;
		return 0;
	}
	
	return CDialog::WindowProc(message, wParam, lParam);
}

BOOL CVolumeMeterDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_pcProgress1.SetRange(0,100);
	m_pcProgress2.SetRange(0,100);

	m_pSoundReceiver->Start();
	m_pSoundProvider = m_pSoundReceiver->CreateMediaProvider(m_pPipe);
	m_pVolumeProbe->Start( m_pPipe, m_pSoundReceiver->GetBlockSize(), this );

	*m_ppMe = this;
	m_bInTransition = false;

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CVolumeMeterDialog::OnCloseButton() 
{
	OnClose();
}

void CVolumeMeterDialog::FreeInternalObjects()
{
	delete m_pSoundReceiver;
	delete m_pVolumeProbe;
	delete m_pPipe;
}

void CVolumeMeterDialog::OnClose() 
{
	m_pSoundReceiver->Stop();
}
