/*
 * MPEG Recorder
 * Copyright (c) 2004 Gadi Bergman
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: VolumeProbe.cpp,v 1.1 2004/04/12 15:36:49 gadibergman Exp $
 */

/************************************************************
			VolumeProbe.cpp
			===============

  A thread that calculates the volume for a stereo 16 bits
  raw sound input
  
************************************************************/

// PCH includes
#include "stdafx.h"

// C includes
#include <iostream.h>
#include <math.h>
#include <process.h>

// Project includes
#include "Common/MutexProtection.h"
#include "TimeKeepingReaderWriter.h"
#include "VolumeProbe.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CVolumeProbe::CVolumeProbe():
m_pReader(NULL),
m_pwDisplayWindow(NULL),
m_uiBlockSize(0),
m_pSamplesBlock(NULL),
m_hTermination(CreateEvent(NULL, TRUE, TRUE, NULL))
{
}

CVolumeProbe::~CVolumeProbe()
{
	// Wait for the thread to exit
	DWORD dwResult = WaitForSingleObject(m_hTermination, INFINITE);
	ASSERT(dwResult == WAIT_OBJECT_0);

	CloseHandle(m_hTermination);

	delete [] m_pSamplesBlock;
}

void CVolumeProbe::Start(TimeKeepingReader *pReader, unsigned int uiBlockSize, CWnd *pWnd)
{
	m_pReader = pReader;
	m_pwDisplayWindow = pWnd;
	m_uiBlockSize = uiBlockSize;
	m_pSamplesBlock = new BYTE[ uiBlockSize ];

	_beginthread(CVolumProbe_ThreadFunc, 0, this);
}

static void CVolumProbe_ThreadFunc(LPVOID pParam) 
{
	CVolumeProbe * pVolumeProbe = (CVolumeProbe*) pParam;
	ThreadCompletionAnnouncement announcement(pVolumeProbe->m_hTermination);

	try {
		pVolumeProbe->Probe();
	}
	catch (std::string * errMess) {
		cerr << errMess << endl;
		AfxMessageBox(_T(errMess->c_str()));
		delete errMess;
		return;
	}
	catch (...) {
		AfxMessageBox(_T("Unhandled exception in Encoding thread"));
		return;
	}
}

void CVolumeProbe::Probe()
{
	while (true)
	{
		Timestamp timestamp;
		Duration duration;

		unsigned int uiBytesRead = m_pReader->ReadBytes( m_pSamplesBlock, m_uiBlockSize, timestamp, duration );

		if ( !uiBytesRead )
			break;

		enum {
			NUM_CHANNELS = 2,
			BYTES_PER_SAMPLE = sizeof(short)
		};
		
		int iNumSamples = uiBytesRead / ( NUM_CHANNELS * BYTES_PER_SAMPLE );

		unsigned int uivChannelVU[NUM_CHANNELS];

		for ( int iChannel = 0; iChannel < NUM_CHANNELS; ++iChannel )
		{
			short* psSampleLevel = (short*) m_pSamplesBlock;
			psSampleLevel += iChannel;
			double dLogLevelSum(0);

			for ( int iSampleNum = 0; iSampleNum < iNumSamples; ++iSampleNum )
			{
				short sSampleLevel = *psSampleLevel;
				dLogLevelSum += log( (long) sSampleLevel * sSampleLevel + 1 );
			
				psSampleLevel += NUM_CHANNELS;
			}

			uivChannelVU[iChannel] = 100 * dLogLevelSum / ( iNumSamples * 32 * log(2) );

		} // for each channel

		m_pwDisplayWindow->PostMessage( VOLUME_UNIT_READY, uivChannelVU[0], uivChannelVU[1] );

	} // while(true)
	m_pwDisplayWindow->PostMessage( END_OF_STREAM, 0, 0 );
}
