/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is MPEG4IP.
 * 
 * The Initial Developer of the Original Code is Cisco Systems Inc.
 * Portions created by Cisco Systems Inc. are
 * Copyright (C) Cisco Systems Inc. 2000, 2001.  All Rights Reserved.
 * 
 * Contributor(s): 
 *		Dave Mackie		dmackie@cisco.com
 *		Bill May 		wmay@cisco.com
 */

// PCH Includes
#include "StdAfx.h"

#include "mp4live.h"
//#include "video_v4l_source.h"
//#include "audio_oss_source.h"

CLiveConfig::CLiveConfig(
	SConfigVariable* variables, 
	config_index_t numVariables, 
	const char* defaultFileName)
: CConfigSet(variables, numVariables, defaultFileName) 
{
	m_appAutomatic = false;
	m_videoCapabilities = NULL;
	m_videoEncode = true;
	m_videoPreviewWindowId = 0;
	m_videoMaxWidth = 768;
	m_videoMaxHeight = 576;
	m_videoNeedRgbToYuv = false;
	m_videoMpeg4ConfigLength = 0;
	m_videoMpeg4Config = NULL;
	m_videoMaxVopSize = 128 * 1024;
	m_audioCapabilities = NULL;
	m_audioEncode = true;
	m_recordEstFileSize = 0;
}

CLiveConfig::~CLiveConfig()
{
	delete m_videoCapabilities;
	delete m_audioCapabilities;
	CHECK_AND_FREE(m_videoMpeg4Config);
}

// recalculate derived values
//DEL void CLiveConfig::Update() 
//DEL {
//DEL 	UpdateVideo();
//DEL 	UpdateAudio();
//DEL 	UpdateRecord();
//DEL }

//DEL void CLiveConfig::UpdateFileHistory(const char* fileName)
//DEL {
//DEL 	u_int8_t i;
//DEL 	u_int8_t end = NUM_FILE_HISTORY - 1;
//DEL 
//DEL 	// check if fileName is already in file history list
//DEL 	for (i = 0; i < end; i++) {
//DEL 		if (!strcmp(fileName, GetStringValue(CONFIG_APP_FILE_0 + i))) {
//DEL 			end = i; 
//DEL 		}
//DEL 	}
//DEL 
//DEL 	// move all entries down 1 position
//DEL 	for (i = end; i > 0; i--) {
//DEL 		SetStringValue(CONFIG_APP_FILE_0 + i, 
//DEL 			GetStringValue(CONFIG_APP_FILE_0 + i - 1));
//DEL 	}
//DEL 
//DEL 	// put new value in first position
//DEL 	SetStringValue(CONFIG_APP_FILE_0, fileName);
//DEL }

//DEL void CLiveConfig::UpdateVideo() 
//DEL {
//DEL 	m_videoEncode =
//DEL 		GetBoolValue(CONFIG_VIDEO_ENCODED_PREVIEW)
//DEL 		/*|| GetBoolValue(CONFIG_RTP_ENABLE)*/
//DEL 		|| (GetBoolValue(CONFIG_RECORD_ENABLE)
//DEL 			&& GetBoolValue(CONFIG_RECORD_ENCODED_VIDEO));
//DEL 
//DEL 	CalculateVideoFrameSize();
//DEL 
//DEL 	GenerateMpeg4VideoConfig(this);
//DEL }

//DEL void CLiveConfig::CalculateVideoFrameSize()
//DEL {
//DEL 	u_int16_t frameHeight;
//DEL 	float aspectRatio = GetFloatValue(CONFIG_VIDEO_ASPECT_RATIO);
//DEL 
//DEL 	// crop video to appropriate aspect ratio modulo 16 pixels
//DEL 	if ((aspectRatio - VIDEO_STD_ASPECT_RATIO) < 0.1) {
//DEL 		frameHeight = GetIntegerValue(CONFIG_VIDEO_RAW_HEIGHT);
//DEL 	} else {
//DEL 		frameHeight = (u_int16_t)(
//DEL 			(float)GetIntegerValue(CONFIG_VIDEO_RAW_WIDTH) 
//DEL 			/ aspectRatio);
//DEL 
//DEL 		if ((frameHeight % 16) != 0) {
//DEL 			frameHeight += 16 - (frameHeight % 16);
//DEL 		}
//DEL 
//DEL 		if (frameHeight > GetIntegerValue(CONFIG_VIDEO_RAW_HEIGHT)) {
//DEL 			// OPTION might be better to insert black lines 
//DEL 			// to pad image but for now we crop down
//DEL 			frameHeight = GetIntegerValue(CONFIG_VIDEO_RAW_HEIGHT);
//DEL 			if ((frameHeight % 16) != 0) {
//DEL 				frameHeight -= (frameHeight % 16);
//DEL 			}
//DEL 		}
//DEL 	}
//DEL 
//DEL 	m_videoWidth = GetIntegerValue(CONFIG_VIDEO_RAW_WIDTH);
//DEL 	m_videoHeight = frameHeight;
//DEL 
//DEL 	m_ySize = m_videoWidth * m_videoHeight;
//DEL 	m_uvSize = m_ySize / 4;
//DEL 	m_yuvSize = (m_ySize * 3) / 2;
//DEL }

//DEL void CLiveConfig::UpdateAudio() 
//DEL {
//DEL 	m_audioEncode =
//DEL 		/*GetBoolValue(CONFIG_RTP_ENABLE)
//DEL 		||*/ (GetBoolValue(CONFIG_RECORD_ENABLE)
//DEL 			&& GetBoolValue(CONFIG_RECORD_ENCODED_AUDIO));
//DEL }

//DEL void CLiveConfig::UpdateRecord() 
//DEL {
//DEL 	u_int64_t videoBytesPerSec = 0;
//DEL 
//DEL 	if (GetBoolValue(CONFIG_VIDEO_ENABLE)) {
//DEL 		if (GetBoolValue(CONFIG_RECORD_RAW_VIDEO)) {
//DEL 			videoBytesPerSec += (u_int64_t)
//DEL 				(((m_videoWidth * m_videoHeight * 3) / 2)
//DEL 				* GetFloatValue(CONFIG_VIDEO_FRAME_RATE) + 0.5);
//DEL 		}
//DEL 		if (GetBoolValue(CONFIG_RECORD_ENCODED_VIDEO)) {
//DEL 			videoBytesPerSec +=
//DEL 				(GetIntegerValue(CONFIG_VIDEO_BIT_RATE) * 1000) / 8;
//DEL 		}
//DEL 	}
//DEL 
//DEL 	u_int64_t audioBytesPerSec = 0;
//DEL 
//DEL 	if (GetBoolValue(CONFIG_AUDIO_ENABLE)) {
//DEL 		if (GetBoolValue(CONFIG_RECORD_RAW_AUDIO)) {
//DEL 			audioBytesPerSec +=
//DEL 				GetIntegerValue(CONFIG_AUDIO_SAMPLE_RATE) 
//DEL 				* GetIntegerValue(CONFIG_AUDIO_CHANNELS)
//DEL 				* sizeof(u_int16_t);
//DEL 		}
//DEL 		if (GetBoolValue(CONFIG_RECORD_ENCODED_AUDIO)) {
//DEL 			audioBytesPerSec +=
//DEL 				(GetIntegerValue(CONFIG_AUDIO_BIT_RATE)) / 8;
//DEL 		}
//DEL 	}
//DEL 
//DEL 	/*
//DEL 	u_int64_t duration = GetIntegerValue(CONFIG_APP_DURATION) 
//DEL 		* GetIntegerValue(CONFIG_APP_DURATION_UNITS);
//DEL 		*/
//DEL 	
//DEL 	m_recordEstFileSize = (u_int64_t) -1L;
//DEL 	/*
//DEL 	m_recordEstFileSize = (u_int64_t)
//DEL 		((double)((__int64)(videoBytesPerSec + audioBytesPerSec) * duration) * 1.025);
//DEL 		*/
//DEL }

//DEL bool CLiveConfig::IsOneSource()
//DEL {
//DEL 	bool sameSourceType =
//DEL 		!strcasecmp(GetStringValue(CONFIG_VIDEO_SOURCE_TYPE),
//DEL 			GetStringValue(CONFIG_AUDIO_SOURCE_TYPE));
//DEL 
//DEL 	if (!sameSourceType) {
//DEL 		return false;
//DEL 	}
//DEL 
//DEL 	bool sameSourceName =
//DEL 		!strcmp(GetStringValue(CONFIG_VIDEO_SOURCE_NAME),
//DEL 			GetStringValue(CONFIG_AUDIO_SOURCE_NAME));
//DEL 
//DEL 	return sameSourceName;
//DEL }

//DEL bool CLiveConfig::IsCaptureVideoSource()
//DEL {
//DEL 	const char *sourceType =
//DEL 		GetStringValue(CONFIG_VIDEO_SOURCE_TYPE);
//DEL 
//DEL 	return !strcasecmp(sourceType, VIDEO_SOURCE_V4L);
//DEL }

//DEL bool CLiveConfig::IsCaptureAudioSource()
//DEL {
//DEL 	const char *sourceType =
//DEL 		GetStringValue(CONFIG_AUDIO_SOURCE_TYPE);
//DEL 
//DEL 	return !strcasecmp(sourceType, AUDIO_SOURCE_OSS);
//DEL }

//DEL bool CLiveConfig::IsFileVideoSource()
//DEL {
//DEL 	const char *sourceType =
//DEL 		GetStringValue(CONFIG_VIDEO_SOURCE_TYPE);
//DEL 
//DEL 	return !strcasecmp(sourceType, FILE_SOURCE);
//DEL }

//DEL bool CLiveConfig::IsFileAudioSource()
//DEL {
//DEL 	const char *sourceType =
//DEL 		GetStringValue(CONFIG_AUDIO_SOURCE_TYPE);
//DEL 
//DEL 	return !strcasecmp(sourceType, FILE_SOURCE);
//DEL }

